$!
$! FILE_ID_TO_NAME.COM
$!
$!	P1:	File number
$!	P2:	Disk device
$!
$	call file_id_to_name 'p1' 'p2'
$	write sys$output "File specification: ",filespec
$	delete/symbol/global filespec
$	delete/symbol/global filename
$	exit
$!
$ file_id_to_name: subroutine
$	old_verify = f$verify('f$trnlnm("file_id_to_name$verify")'+0)
$!
$! Given a file ID and disk device name, find out the file name
$!
$!	P1: File number from File ID (decimal, or hex with %X prefix)
$!	P2: Disk device on which this file is located
$!
$!	Return file specification in global symobl 'filespec'
$!
$	if .not. f$getdvi("''p2'","EXISTS")
$	then
$		write sys$output "    Can't find device ''p2'"
$		goto error_exit
$	endif
$	if f$locate(":",p2) .ne. f$length(p2) then p2 = p2 - ":"
$! Initialization
$	debug = "!"	! "!" to disable debug messages; "" to enable messages
$	pid = f$getjpi("","PID")
$	temp_file := file_id_to_name_temp_'pid'.temp_file
$	maxfiles = f$getdvi("''p2'","MAXFILES")
$	cluster = f$getdvi("''p2'","CLUSTER")
$! Look up the file itself
$	call lookup_file 'p1' 'p2'
$	if filename .eqs. "?.?;?" then goto error_exit
$	initial_filename = filename	!Remember this filespec
$! Look up its directory tree
$	filespec = "]" + filename
$ dir_loop:
$	call lookup_file 'dir_file_id' 'p2'
$	if filename .eqs. "?.?;?" then goto error_exit
$	if filename .eqs. "000000.DIR;1" then goto dir_done
$! If we're just backtracking from extension file headers, keep on looking
$	if filename .eqs. initial_filename .and. -
    f$locate(".DIR;1",filename) .eq. f$length(filename) then goto dir_loop
$	dot = "."
$	if f$extract(0,1,filespec) .eqs. "]" then dot = ""
$	filespec = (filename - ".DIR;1") + dot + filespec
$	goto dir_loop
$ dir_done:
$	if f$extract(0,1,filespec) .eqs. "]" then -
		filespec = "000000" + filespec	!File is in the MFD
$	filespec == p2 + ":[" + filespec
$!!!	write sys$output "    File specification: ",filespec
$ 	goto done
$ error_exit:
$	filespec == ""
$ done:
$	if old_verify then set verify
$	endsubroutine
$!
$! Look up a file by File ID and return its name and the File ID of the
$! directory it is in
$!
$ lookup_file: subroutine
$	x = ((maxfiles+4095) / 4096) + (4 * cluster) + 'p1'
$	dump/file_header/block=(start:'x',count:1)/output='temp_file' -
		'p2':[000000]indexf.sys
$'debug' type/page 'temp_file';
$	define/user sys$error _nla0:
$	define/user sys$output _nla0:
$	search 'temp_file' "File name:","Back link file identification:"-
		/out='temp_file'_1
$	search_severity = $severity	!Save search result: success=1, fail=3
$	delete 'temp_file';*	!Delete temporary file
$	if search_severity .eq. 3 then goto err	!Didn't find the info we need ->
$	close/nolog temp
$'debug' type/page 'temp_file'_1;
$	open/read temp 'temp_file'_1
$! Read the Directory Backlink record
$	read/end=eof temp rec
$	rec = f$edit(rec,"compress,trim")
$	if f$extract(0,9,rec) .eqs. "Back link"
$	then
$	    dir_backlink = f$element(4," ",rec) - "(" - ")"
$'debug' show symbol dir_backlink	!!!
$	    dir_file_id == f$element(0,",",dir_backlink)	!File number
$'debug' show symbol dir_file_id	!!!
$	else
$	    goto err
$	endif
$! Read the File Name record
$	read/end=eof temp rec
$	rec = f$edit(rec,"compress,trim")
$	if f$extract(0,10,rec) .eqs. "File name:"
$	then
$	    filename == f$element(2," ",rec)
$'debug' show symbol filename	!!!
$	else
$	    goto err
$	endif
$	close temp
$	delete 'temp_file'_1;*
$	goto xit	!Normal exit -->
$ eof:	!Premature end-of-file
$	close temp
$ err:	!Error occurred which prevented looking up the filename and/or directory
$	write sys$output "    Error looking up File Number ",p1," on disk ",p2
$	dir_file_id == "[?]"
$	filename == "?.?;?"
$ xit:
$	endsubroutine
