$	if f$mode() .eqs. "BATCH" then set noverify
$!
$! LOCKTIME.COM
$!
$!	Measure lock-request latencies in a cluster, by timing actual lock
$!	requests.
$!
$!	To start with, run this procedure with a P1 parameter of "BUILD"
$!	to assemble/compile/link the LOCKTIME_* Macro-32 program pair.  Do
$!	this once on each architecture in a mixed-architecture, on the node
$!	which is running the oldest version of VMS for each architecture.
$!
$!	This procedure requires a batch queue on each node in the cluster,
$!	with JOB_LIMIT set high enough that we can run 2 additional jobs in
$!	each queue simultaneously.  So next, edit this procedure to reflect
$!	the batch queue naming convention in your cluster.  For example,
$!	if your queues are named SYS$BATCH_nodename, you'd use:
$!		$	queue_prefix = "SYS$BATCH_"
$!		$	queue_suffix = ""
$!	If your naming convention were nodename_BATCH, you'd use:
$!		$	queue_prefix = ""
$!		$	queue_suffix = "_BATCH"
$!	Edit the following two lines to reflect your configuration:
$	queue_prefix = "NEEDS TO BE EDITED"
$	queue_suffix = "NEEDS TO BE EDITED"
$!
$!	Testing requires running under a username with an ENQLM quota of a
$!	little over 5000 (unless you edit the OP_COUNT symbol in both
$!	LOCKTIME_1.MAR and LOCKTIME_3A.MAR and rebuild the programs).
$!
$!	To execute the lock-latency tests, simply run this procedure without
$!	any parameters.  When testing is completed, the results will be
$!	available in the file LTDATA.CSV.  This is a comma-separated-variable
$!	format file suitable for upload into a Microsoft Excel spreadsheet.
$!	Alternatively, results are also available in the LTDATA.LOG file, in
$!	a human-readable format.
$!
$!	If you have problems with this procedure, check the website
$!	http://encompasserve.org/~parris/ for a later version; failing that,
$!	send me e-mail and I'll try to help.  Suggestions and improved
$!	versions are always welcome, of course.
$!
$! Author: Keith Parris  parris@encompasserve.org or keithparris@yahoo.com
$!                       or keith.parris@hp.com
$! http://www.geocities.com/keithparris/ or http://encompasserve.org/~parris/
$! or http://www2.openvms.org/parris/
$!
$!	Version is:					V1.0 March 11, 2003
$!
$! Modification history:
$!
$!	V1.0 March 11, 2003	 Initial version, based on combining several
$!			 	 existing DCL procedures and Roy's programs
$!
$!	The LOCKTIME_1.MAR and LOCKTIME_3.MAR programs were written by
$!	Roy G. Davis, author of VAXcluster Principles (ISBN 1-55558-112-9).
$!	They were ported to Alpha by Christian Moser.  LOCKTIME_3A is
$!	LOCKTIME_3 with some simple modifications by Keith Parris so it can
$!	be run in batch.
$!
$!-----------------------------------------------------------------------------
$! We call/submit ourselves recursively.  Find out what procedure file this is.
$       proc = f$environment("procedure")
$! Find out the device and directory where this procedure is located.  Some
$! related files will also be located here.
$       proc_dir = f$parse(proc,,,"DEVICE") + f$parse(proc,,,"DIRECTORY")
$	delete := delete
$! Validate the P1 parameter
$	if p1 .nes. "LT_CLIENT" .and. p1 .nes. "LT_MASTER" .and. -
           p1 .nes. "LTTEST" .and. p1 .nes. "LTDATA" .and. -
           p1 .nes. "LTCSV" .and. p1 .nes. "BUILD" .and. p1 .nes. ""
$	then
$		write sys$output "Error in P1 parameter: ''p1'"
$		exit
$	endif
$!
$! If we're doing a BUILD, or calling ourselves recursively, go to the
$! appropriate code section
$!
$	if p1 .nes. "" then goto 'p1'
$!
$! Make sure they've built the programs first
$!
$	if (f$search("''proc_dir'LOCKTIME_1_*.EXE")  .eqs. "") .or. -
	   (f$search("''proc_dir'LOCKTIME_3A_*.EXE") .eqs. "")
$	then
$		write sys$output -
"The LOCKTIME* program pair needs to be built.  Execute $ @LOCKTIME BUILD"
$		write sys$output -
"once on each architecture in the cluster, on the oldest version of VMS"
$		write sys$output -
"running in each case.  This cluster contains the following architecture(s)"
$		write sys$output -
"and version(s):"
$		mcr sysman set environment/cluster
do write sys$output f$getsyi("ARCH_NAME")," ",f$getsyi("VERSION")
exit
$		exit
$	endif
$!
$! Make sure they've customized the queue name data
$!
$	if queue_prefix .eqs. "NEEDS TO BE EDITED" .or. -
           queue_suffix .eqs. "NEEDS TO BE EDITED"
$	then
$		write sys$output -
"LOCKTIME.COM needs to be edited to reflect the batch queue naming convention"
$		write sys$output -
"for the nodes in your cluster.  To run the tests it performs, it needs a"
$		write sys$output -
"batch queue on each node which has a /JOB_LIMIT value high enough to allow"
$		write sys$output -
"for at least 2 additional jobs to run simultaneously in the queue."
$		write sys$output ""
$		write sys$output -
"Based on the batch queues found for this node, possible candidate batch queue"
$		write sys$output -
"name formats may include the following.  (After each prospective queue name"
$		write sys$output -
"format found is the DCL code to insert into LOCKTIME.COM to use those queues.)"
$!!		write sys$output ""
$!!		nodename = f$edit(f$getsyi("NODENAME"),"TRIM")
$!!		show queue/batch/noall *'nodename'*
$!
$! Look for possibly-suitable batch queues using this node as an example
$!
$		nodename = f$edit(f$getsyi("NODENAME"),"TRIM")
$		x = f$getqui("CANCEL_OPERATION")	!Clear any previous operation
$		qns = ""
$ q_loop:
$		qn = f$getqui("DISPLAY_QUEUE", "QUEUE_NAME","*''nodename'*","BATCH")
$		if qn .eqs. "" then goto got_qs	!No more batch queues
$		qns = qns + "," + qn		!Record the name of this queue
$		goto q_loop
$ got_qs:	i = 0	!Now look at each candidate queue
$ qn_loop:	i = i + 1
$		qn = f$element(i,",",qns)
$		if qn .eqs. "" .or. qn .eqs. "," then exit
$		jl= f$getqui("DISPLAY_QUEUE", "JOB_LIMIT", qn)
$		write sys$output "  Queue ",qn,"; /JOB_LIMIT is ",jl
$		t = f$locate(nodename,qn)
$		queue_prefix = f$extract(0,t,qn)
$		queue_suffix = f$extract(t+f$length(nodename),99,qn)
$!! show symbol queue_prefix
$!! show symbol queue_suffix
$		write sys$output "    $ queue_prefix = """,queue_prefix,""""
$		write sys$output "    $ queue_suffix = """,queue_suffix,""""
$		goto qn_loop
$	endif
$!----------------------------------------------------------------------------
$!
$! Mainline routine
$!
$	temp_file := LOCKTIME_TEMP.TEMP_FILE
$!
$!!! show time
$!
$! Run the tests
$!
$	@'proc'/out='temp_file'_lttest_log LTTEST 3    !P2 parameter is # of samples to take
$! Check for errors
$	search 'temp_file'_lttest_log error,"-e-","-f-","-w-"
$	if $severity .eq. 3 then -
		write sys$output "Testing appears to have been successful."
$!
$! Parse the log file to extract the data
$!
$	@'proc'/out=LTDATA.LOG LTDATA 'temp_file'_lttest_log
$!! type/page ltdata.log	!!!
$!
$!! Sort the nodenames in alphabetical order
$!!	sort/key=(pos:1,size:6)/key=(pos:12,size=6) -
$!!_!		'temp_file'_lttest_log 'temp_file'_ltdata_srt
$!! We create the nodename list itself in alphabetical order now, so the data
$!! should already be sorted.  This also makes it easier to track the progress
$!! of the test jobs, as they also progress through nodes by nodename in
$!! alphabetical order.
$!
$! Create a Comma-Separated Variables (CSV) format file suitable for
$! creating an Excel spreadsheet
$	@'proc' LTCSV LTDATA.LOG
$!! type ltdata.csv	!!!
$	write sys$output -
"Lock latency results are available in the files LTDATA.CSV and LTDATA.LOG."
$	delete 'temp_file'_lttest_log;*
$	exit
$!----------------------------------------------------------------------------
$!
$! Routine to drive the tests
$!
$ LTTEST:
$!
$! LTTEST.COM
$!
$!	Run LOCKTIME_1/_3a to obtain lock-request latency values
$!
$!	P2 = Number of tests to perform in each case (default=3)
$!
$! (By doing more than one test, and averaging the results, the intention is
$!  to provide more statistically-valid results.)
$!
$! Queue names are assumed to contain the nodename somewhere.
$! The following strings define a prefix and/or suffix for the batch queue
$! naming convention in a particular cluster.  (Make sure these queues have a
$! high enough value for JOB_LIMIT so that 2 of our jobs can be running
$! in a given queue at the same time.)
$!!	queue_prefix = ""
$!!	queue_suffix = "$batch"
$!! These are defined in the mainstream procedure now.
$!
$! Specify the delay between test samples
$!
$!	inter_test_delay := 0:0:10
$	inter_test_delay := 0:0:1
$!
$	test_cases = f$integer(p2)
$	if test_cases .le. 0 then test_cases = 3
$!
$! Obtain a list of cluster nodes, in alphabetical order by nodename
$!
$	context = ""
$	node_cnt = 0
$ node_list_loop:
$	csid = f$csid(context)	!Get the CSID of the next cluster member
$	if csid .eqs. "" then goto node_list_done
$	nodename = f$edit(f$getsyi("NODENAME",,csid),"TRIM")
$! Add nodename to list, but in alphabetical order (a linear insertion sort)
$	if node_cnt .eq. 0
$	then	!Starting the node list
$	    node_list = "," + nodename
$	else	!Insert nodename at appropriate spot in the existing list
$	    i = 1
$	    temp_list = ""
$ insert_loop:
$	    if nodename .lts. f$element(i,",",node_list)
$	    then	!Insert new node before this element
$		temp_list = temp_list + "," + nodename
$ add_rest:	if i .le. node_cnt
$		then
$		    temp_list = temp_list + "," + f$element(i,",",node_list)
$		    i = i + 1
$		    goto add_rest
$		endif
$		goto insert_done
$	    else	!Haven't found the right spot yet
$		temp_list = temp_list + "," + f$element(i,",",node_list)
$	    endif
$	    i = i + 1
$	    if i .le. node_cnt then goto insert_loop
$!Ran out of list; add at end
$	    temp_list = temp_list + "," + nodename
$ insert_done:
$	    node_list = temp_list
$	endif
$!
$	node_cnt = node_cnt + 1
$	goto node_list_loop
$!
$ node_list_done:
$!! show symbol node_cnt	!!!
$!! show symbol node_list	!!!
$!
$! Now we have a list of all the nodes in the cluster, in alphabetical order.
$!
$! Run tests with each node as the lock-master node in turn.  While a given node
$! is the lock-master, we then run tests with each node as a client node.
$!
$! So when we get done, we have lock-latency data for every possible
$! combination of nodes as master and client nodes.
$!
$	i = 0	!Index into node_list for Master nodes
$ master_loop:
$	i = i + 1	!Point to next Master node
$	master_node = f$element(i,",",node_list)
$	write sys$output "Master node: ",master_node
$	submit/noprint/queue='queue_prefix''master_node''queue_suffix'-
	    /name="LT_Master"/param=LT_MASTER -
	    /log='proc_dir'LT_MASTER_'master_node'.LOG 'proc'
$	LT_MASTER_job = $entry
$!
$! It would be better to synchronously signal somehow that all the locks have
$! been acquired (perhaps waiting here until the last one of the 5,000
$! locks has been acquired by the master node), but here we just wait for a
$! fixed amount of time.
$ wait 0:0:30	!Wait for LOCKTIME_3a to acquire the locks on 'master_node'
$!
$! Perform the test for the specified number of iterations
$	t = 0	!Count the test iterations
$ test_loop:
$!
$! With the present lock-master node, run the client program on each node
$! in the cluster (including the lock-master node itself -- that will give us
$! local lock latency timings, too).
$	j = 0	!Index into node_list for Client nodes
$ client_loop:
$	j = j + 1	!Point to next Client node
$	client_node = f$element(j,",",node_list)
$	write sys$output "   Client: ",client_node
$	submit/noprint/queue='queue_prefix''client_node''queue_suffix'-
	    /name="LT_Client"/param=(LT_CLIENT,'client_node','master_node')-
	    /log='proc_dir'LT_CLIENT_'client_node'_'master_node'.LOG 'proc'
$	LT_CLIENT_job = $entry
$ client_wait:
$	status = f$getqui("DISPLAY_ENTRY", "JOB_STATUS", 'LT_CLIENT_job')
$	if status .nes. ""
$	then
$		write sys$output "   Client job status is ",status
$		wait 0:0:1	!Wait a second
$		goto client_wait
$	endif
$! The measurement job has finished.  Check the results.
$	write sys$output "Master node: ",master_node,-
		", Client node: ",client_node,".  Results were:"
$	search/wind=(0,1) 'proc_dir'lt_client_'client_node'_'master_node'.log -
		"lock conversion -"
$! If the client job was successful,
$! Clean up the log file from the Client job
$	if $severity .eq. 1
$	then
$	    delete 'proc_dir'lt_client_'client_node'_'master_node'.log;/nolog
$	else
$	    write sys$output "*** Error *** ",-
"LT_Client job on node ''client_node' seems to have failed.  See"
$	    write sys$output "*** Error ***   ",-
F$SEARCH("''proc_dir'LT_CLIENT_''client_node'_''master_node'.LOG")
$	endif
$! Do the next client node
$	if j .lt. node_cnt then goto client_loop
$! Do the next test iteration
$	t = t + 1
$	if t .lt. test_cases
$	then
$		wait 'inter_test_delay'	!Wait a bit before repeating the tests
$		goto test_loop
$	endif
$! Get rid of the locks on the master node so we can move mastership to another
$	delete/entry='LT_MASTER_job' 'queue_prefix''master_node''queue_suffix'/log
$! Wait for the job to be aborted
$!!	wait 0:0:10	!Wait for job to abort
$!! Instead of waiting a fixed amount of time, check until the job is gone
$ master_wait:
$	status = f$getqui("DISPLAY_ENTRY", "JOB_STATUS", 'LT_MASTER_job')
$	if status .nes. ""
$	then
$		write sys$output "   Master job status is ",status
$		wait 0:0:1	!Wait a second
$		goto master_wait
$	endif
$! Check the lock-master node logfile for errors.
$! Note that appearance of the line:
$!    %JBC-F-JOBABORT, job aborted during execution
$! is normal and to be expected, since we kill the job ourselves.
$	search 'proc_dir'LT_MASTER_'master_node'.LOG error,"-e-","-w-"	!,"-f-"
$	if $severity .eq. 3
$	then	!Clean up the log file from the Master job
$	    delete 'proc_dir'lt_master_'master_node'.log;
$	else	!Save the log file for analysis, and let the user know
$	    write sys$output "*** Error *** ",-
"LT_Master job on node ''master_node' seems to have failed.  See"
$	    write sys$output "*** Error ***   ",-
F$SEARCH("''proc_dir'LT_MASTER_''master_node'.LOG;")
$	endif
$! Do the next master node
$	if i .lt. node_cnt then goto master_loop
$!
$	exit
$!----------------------------------------------------------------------------
$!
$ LTDATA:
$!
$! LTDATA.COM - Extract data from LTTEST.LOG file
$!
$!	P2: File specification of log file from LTTEST run
$!
$	close/nolog log
$	if p2 .eqs. "" then p2 = "''proc_dir'lttest.log"
$	open/read log 'p2'
$	target_string = "Average time per operation: "
$ ltdata_loop:
$	read/end=ltdata_eof log rec
$	if f$locate("Results",rec) .eq. f$length(rec) then goto ltdata_loop
$! Samples of the type of line we're parsing here:
$!Master node: BBC307, Client node: BBC204.  Results were:
$!Master node: BBC307, Client node: BBC208.  Results were:
$!Master node: BBC307, Client node: BBC307.  Results were:
$	master_node = f$element(2," ",rec) - ","
$!!! show symbol master_node	!!!
$	client_node = f$element(5," ",rec) - "."
$!!! show symbol client_node	!!!
$! Here we have the data as output from the LOCKTIME_1 program
$	read/end=eof log rec	!Read and discard "Lock conversion -" record
$	read/end=eof log rec	!This record contains the data of interest
$!!!	show symbol rec	!!!
$	t = f$locate(target_string,rec) + f$length(target_string)
$! Pick up the latency measurement (it's in scientific notation, e.g.
$! ".1234E+03", in units of microseconds)
$	time = f$element(0," ",f$extract(t,99,rec))
$! show symbol time	!!!
$	pre_decimal = f$element(0,".",time)
$	post_decimal = f$element(0,"E",f$element(1,".",time))
$	exponent = f$element(1,"E",time)
$!!! show sym pre_decimal	!!!
$!!! show sym exponent	!!!
$!!! show sym post_decimal	!!!
$	shift_cnt = f$integer(exponent)
$	increment = f$integer(f$extract(0,1,exponent)+"1")
$!!! show symbol shift_cnt	!!!
$!!! show symbol increment	!!!
$ shift:
$	if shift_cnt .ne. 0
$	then	!Some decimal-point shifting needed
$	    if increment .gt. 0
$	    then	!E+nn means positive exponent; shift decimal point right
$		if f$length(post_decimal) .le. 1 then -
			post_decimal = post_decimal + "0"
$		pre_decimal = pre_decimal + f$extract(0,1,post_decimal)
$		post_decimal = f$extract(1,99,post_decimal)
$	    else	!E-nn means negative exponent; shift decimal point left
$		if f$length(pre_decimal) .le. 1 then -
			pre_decimal = "0" + pre_decimal
$		post_decimal = -
    f$extract(f$length(pre_decimal)-1,1,pre_decimal) + post_decimal
$		pre_decimal = f$extract(0,f$length(pre_decimal-1),pre_decimal)
$	    endif
$	    shift_cnt = shift_cnt - increment
$	    goto shift	!See if more shifting is needed
$	endif
$ remove_leading_zeroes:
$	if f$length(pre_decimal) .gt. 1 .and. -
	  (f$extract(0,1,pre_decimal) .eqs. "0")
$	then
$	    pre_decimal = pre_decimal - "0"
$	    goto remove_leading_zeroes
$	endif
$	shifted_time = pre_decimal + "." + post_decimal
$ write sys$output client_node," --> ",master_node," ",shifted_time," microseconds"
$	goto ltdata_loop
$ ltdata_eof:
$	close log
$	exit
$!----------------------------------------------------------------------------
$!
$ LTCSV:
$!
$! Read an LTDATA.LOG file and create a .CSV file suitable for importing
$! into an Excel spreadsheet
$!
$!	P2 parameter: Filespec of logfile
$!
$	close/nolog log
$	if p2 .eqs. "" then p2 = "''proc_dir'ltdata.log"
$	open/read log 'p2'	!ltdata.srt	!ltdata.log
$! What we're parsing looks like this:
$! BBC309 --> BBC309 3.6000 microseconds
$! BBC309 --> BBC309 4.0000 microseconds
$! BBC309 --> BBC309 5.8000 microseconds
$! BBC305 --> BBC309 363.60 microseconds
$! BBC305 --> BBC309 326.60 microseconds
$! etc.
$	node_list = ","
$ ltcsv_loop:
$	read/end=ltcsv_eof log rec
$!!! show symbol rec	!!!
$	if f$element(1," ",rec) .nes. "-->" .or. -
	   f$element(4," ",rec) .nes. "microseconds" then goto ltcsv_loop
$	client_node = f$element(0," ",rec)
$	master_node = f$element(2," ",rec)
$	microseconds = f$element(3," ",rec)
$! Might as well remove trailing zeroes to save symbol space and reduce the
$! chance of overflowing DCL's symbol-length limit in large clusters
$	if f$locate(".",microseconds) .ne. f$length(microseconds)
$	then
$ unzero_loop:	if f$extract(f$length(microseconds)-1,1,microseconds) .eqs. "0"
$		then
$			microseconds = -	!Remove trailing zero
			    f$extract(0,f$length(microseconds)-1,microseconds)
$		goto unzero_loop
$		endif
$	endif
$! Remove a trailing decimal point if present
$	if f$extract(f$length(microseconds)-1,1,microseconds) .eqs. "." then -
		microseconds = -	!Remove trailing period
			    f$extract(0,f$length(microseconds)-1,microseconds)
$!!! show symbol client_node	!!!
$!!! show symbol master_node	!!!
$!!! show symbol microseconds	!!!
$	if f$type(lt_'client_node'_'master_node') .eqs. "" then -
		lt_'client_node'_'master_node' = ""
$	lt_'client_node'_'master_node' = lt_'client_node'_'master_node' + -
		"," + microseconds
$! See if the node(s) are in our list yet
$	if f$locate(","+client_node+",",node_list) .eq. f$length(node_list) then -
		node_list = node_list + client_node + ","
$	if f$locate(","+master_node+",",node_list) .eq. f$length(node_list) then -
		node_list = node_list + master_node + ","
$	goto ltcsv_loop
$!
$ ltcsv_eof:
$	close log
$!!! show symbol node_list
$! Create the .CSV format output file
$	close/nolog csv
$	open/write csv ltdata.csv
$! Create the top line (node headings)
$	title_line = ""
$	i = 1
$ head_loop:
$	node = f$element(i,",",node_list)
$!!! show symbol node	!!!
$	if node .eqs. "," .or. node .eqs. "" then goto head_done
$	title_line = title_line + ",""" + node + """"
$	i = i + 1
$	goto head_loop
$ head_done:
$!!! show symbol title_line	!!!
$	write/symbol csv title_line
$!
$! Now create one row for each node
$	i = 1
$ row_loop:
$	client_node = f$element(i,",",node_list)
$	if client_node .eqs. "," .or. client_node .eqs. "" then goto done
$	line = """" + client_node + """"  !Name of node in left-hand column
$	j = 1
$ col_loop:
$	master_node = f$element(j,",",node_list)
$	if master_node .eqs. "," .or. master_node .eqs. "" then goto col_done
$	if f$type(lt_'client_node'_'master_node') .eqs. ""
$	then
$		latency = ""	!No data for this point
$		if client_node .eqs. master_node then latency = """\"""
$	else	!Create a formula to calculate the average latency
$		latency = """=average(" + -
			(lt_'client_node'_'master_node'-",") + ")"""
$	endif
$	line = line + "," + latency
$	j = j + 1
$	goto col_loop
$ col_done:
$	write/symbol csv line
$	i = i + 1
$	goto row_loop
$ done:
$	close csv
$	exit
$!----------------------------------------------------------------------------
$!
$! Run LOCKTIME_1 on a client node
$!
$! P2: Name of client node on which this is (supposed to be) running
$! P3: Name of lock-master node while this job is running
$!
$! LOCKTIME_1 measures lock latency, reports the results, and exits.
$!
$ LT_CLIENT:
$	if f$mode() .eqs. "BATCH" then set noverify
$       proc = f$environment("procedure")
$       proc_dev = f$parse(proc,,,"DEVICE")
$       proc_dir = proc_dev + f$parse(proc,,,"DIRECTORY")
$	node = f$edit(f$getsyi("NODENAME"),"TRIM")
$	if node .nes. p2
$	then
$		write sys$output -
			"*** Error *** LT_CLIENT: Client job for node ",p2,-
			" is actually running on node ''node'."
$		exit
$	endif
$	write sys$output "Running on node ",node," with lock-master node ",p3
$	arch = f$getsyi("ARCH_NAME")
$	run 'proc_dir'locktime_1_'arch'.exe
$	write sys$output "LT_CLIENT done..."
$	exit
$!----------------------------------------------------------------------------
$!
$! Acquire and hold the set of locks used for lock-latency tests.  This
$! job retains the lock-mastership of the set of locks for the duration of
$! a set of client tests.  Then we kill this job with $DELETE/ENTRY.
$!
$! Run LOCKTIME_3A to hold the locks on a node, making it the lock-master
$! for a set of client tests.
$!
$! LOCKTIME_3A takes out and holds a set of locks (5,000 by default) and
$! holds them until we abort this job.
$!
$ LT_MASTER:
$	if f$mode() .eqs. "BATCH" then set noverify
$       proc = f$environment("procedure")
$       proc_dev = f$parse(proc,,,"DEVICE")
$       proc_dir = proc_dev + f$parse(proc,,,"DIRECTORY")
$	arch = f$getsyi("ARCH_NAME")
$ write sys$output "LT_MASTER running on node ",f$getsyi("NODENAME")
$	run 'proc_dir'locktime_3a_'arch'.exe
$! This program is supposed to run "forever"...
$! In the normal case, this job is aborted with a $DELETE/ENTRY command.
$	write sys$output "*** Error *** LT_MASTER: LOCKTIME_3A program exited."
$	exit
$!----------------------------------------------------------------------------
$!
$! Build the LOCKTIME_* program pair
$!
$! This procedure creates an executable image for the architecture on which
$! we're running.
$!
$! Because we may run in a mixed-architecture cluster, this procedure will need
$! to be run once on each type of architecture in the cluster. 
$!
$! This program has no dependencies on VMS version, and only makes standard
$! system service calls (requiring no inner-mode code, and is not linked
$! against the system) so just the normal user-mode things apply, like needing
$! to link it on a version of VMS that is no older than the version on which
$! it is to be run (so if you're running multiple version of VMS in the same
$! cluster, to be safe, just run this procedure on the oldest version).
$!
$ BUILD:
$	old_dir = f$environment("DEFAULT")
$	set default 'proc_dir'
$! See if there's a copy of the source out there.  If not, create it.
$	if f$search("locktime_1.mar") .eqs. ""
$	then
$		create locktime_1.mar
$DECK
	.library	/sys$library:lib/
;******************************************************************************
;                                                                             *
;  Program:    LOCKTIME_1                                                     *
;                                                                             *
;  Author:     Roy G. Davis                                                   *
;                                                                             *
;  Date:       August 11, 1993                                                *
;                                                                             *
;  Purpose:    This program is used to determine the average time required    *
;              at process level for common lock operations.  While it employs *
;              multiple resource names at the same time, it holds only one    *
;              lock per per resource name at any given time.                  *
;                                                                             *
;  Summary:    After a small amount of initialization, the program executes   *
;              in succession three loops to (1) acquire one CR lock on each   *
;              member of a set of resource names, (2) convert these locks to  *
;              PR mode, and (3) release the locks.  Each loop is timestamped  *
;              on entry and exit.  The program completes by outputting for    *
;              each loop, and thus for each type of operation, three items of *
;              data:  (1) the number of operations per loop, (2) the amount   *
;              of time spent in each loop, and (3) the average amount of      *
;              time required for the type operation performed within the      *
;              loop.                                                          *
;                                                                             *
;  Parameters: The number of resource names used in this program, and thus    *
;              the number of lock operations per loop, is determined by the   *
;              setting of the OP_COUNT parameter.  For example, to use 5000   *
;              resource names and have 5000 operations performed during each  *
;              loop, the following statement should appear immediately after  *
;              the $LCKDEF macro reference:                                   *
;                                                                             *
;                                 OP_COUNT = 5000                             *
;                                                                             *
;              The ENQ limit for the job from within which this program is    *
;              run should be set to at least the value of OP_COUNT plus the   *
;              maximum number of other locks held by the job while this       *
;              program is running.  If no other locks are held by the job,    *
;              then setting the ENQ limit to OP_COUNT+30 should generally     *
;              be sufficient.                                                 *
;                                                                             *
;  Note:       The set of resource names used by this program consists of a   *
;              sequence of character strings with the format "ROYDAVISxxxx",  *
;              where xxxx is a 4-byte binary number whose value ranges from   *
;              0 to OP_COUNT-1.  The seed for this sequence is in location    *
;              RESOURCE, with xxxx initialized to 0 by four adjacent NULL     *
;              bytes at the end of the seed.  Changing this convention may    *
;              necessitate changes to the loop for acquiring the CR locks.    *
;              In particular, these changes would involve instructions that   *
;              initialize a register pointing to the seed, and that perform   *
;              the arithmetic for generating the sequence of resource names.  *
;                                                                             *
;  Assembly:   To assemble and link this program for execution, simply use    *
;              the following two VMS DCL commands:                            *
;                                                                             *
;                         MACRO name_of_this_program                          *
;                         LINK name_of_this_program                           *
;                                                                             *
;                                                                             *
;  Edit history:	18-aug-93 kloodged to use lib$put_output  ded	      *
;                                                                             *
;                                                                             *
;******************************************************************************

$LCKDEF
; User's ENQLM quota must be at least a bit bigger than the following number:
OP_COUNT = 5000

LF = 10
CR = 13
NULL = 0

	.psect	data	quad,rd,wrt,noexe
;
; Messages indicating when each loop begins and ends
;

NMSG1:      .ASCID      <CR><LF><LF>/Starting new lock acquisition/
NMSG2:      .ASCID      <CR><LF>/New lock acquisition complete/

CMSG1:      .ASCID      <CR><LF><LF>/Starting lock conversion/
CMSG2:      .ASCID      <CR><LF>/Lock conversion complete/
DMSG1:      .ASCID      <CR><LF><LF>/Starting lock dequeuing/
DMSG2:      .ASCID      <CR><LF>/Lock dequeuing complete/

RMSG1:      .ASCID      <CR><LF>/New lock acquisition -/
RMSG2:      .ASCID      <CR><LF>/Lock conversion -/
RMSG3:      .ASCID      <CR><LF>/Lock dequeuing -/

RMSG:       .ADDRESS    RMSG1,RMSG2,RMSG3

RESULT:     .ASCID      <CR><LF>/   Started: /		
RSLT_STRT:  .BLKB       23
            .ASCII      /   Stopped: /
RSLT_STOP:  .BLKB       23
            .ASCII      <CR><LF>/   Number of 100 ns. intervals: /
RSLT_NS:    .BLKB       20
            .ASCII      <CR><LF>/   Number of lock operations: /
RSLT_NOP:   .BLKB       5
            .ASCII      <CR><LF>/   Average time per operation: /
RSLT_USOP:  .BLKB       12
            .ASCII      /us./<CR><LF>
            RSLT_LEN = .-RESULT-2
	    .ASCII 	/   /		; kloodge

RSTR_DES:   .LONG       23
            .ADDRESS    RSLT_STRT
RSTP_DES:   .LONG       23
            .ADDRESS    RSLT_STOP

DES_S_DEL:  .WORD       1
            .BYTE       DSC$K_DTYPE_Q
            .BYTE       DSC$K_CLASS_S
            .ADDRESS    0

DES_D_DEL:  .WORD       1
            .BYTE       DSC$K_DTYPE_D
            .BYTE       DSC$K_CLASS_S
            .ADDRESS    D_FLOAT

DES_D_RNS:  .WORD       20
            .BYTE       DSC$K_DTYPE_T
            .BYTE       DSC$K_CLASS_S
            .ADDRESS    RSLT_NS

DES_USOP:   .WORD       12
            .BYTE       DSC$K_DTYPE_T
            .BYTE       DSC$K_CLASS_S
            .ADDRESS    RSLT_USOP

D_FLOAT:    .D_FLOATING 0
D_FL_DIV:   .D_FLOATING 0
I_FL_DIV:   .QUAD       <10*OP_COUNT>

NOP_CTRL:   .ASCID      /!UL/                       ; FAO control string and
NOP_DESC:   .LONG       5                            ; output descriptor for
            .ADDRESS    RSLT_NOP                     ; lock operation count

EDIV_QUO:   .LONG       0
EDIV_REM:   .LONG       0

DELAY_1:    .ASCID      /0 00:00:01.00/
DELAY_2:    .ASCID      /0 00:00:02.00/
DELAY_3:    .ASCID      /0 00:00:04.00/
WAKE_TIME:  .BLKQ       1

TIME:       .BLKQ       6
TIME_DIFF:  .BLKQ       3

TIME_STR:   .BLKB       23
            .BYTE       <CR>,<LF>
            TIME_LEN = .-TIME_STR
TIME_DESC:  .LONG       23
            .ADDRESS    TIME_STR

TT_NAME:    .ASCID      /SYS$OUTPUT/             	; Terminal output device
TT_CHAN:    .BLKW       1                           ; I/O chan for terminal
TT_IOSB:    .BLKQ       1                           ; IOSB for terminal $QIO

LK_STS_BL:  .BLKQ       1                           ; LKSB for $ENQ
RESOURCE:   .ASCID      /ROYDAVIS/-
                        <NULL><NULL><NULL><NULL>    ; Resource name
LK_ID_TBL:  .BLKL       OP_COUNT                    ; Lock id storage


DES_S_DIV:  .WORD       1                           ; Source descriptor for
            .BYTE       DSC$K_DTYPE_Q                ; integer divisor to
            .BYTE       DSC$K_CLASS_S                ; convert quadword
            .ADDRESS    I_FL_DIV                     ; time delta to D_FLOAT

DES_D_DIV:  .WORD       1                           ; Destination descriptor
            .BYTE       DSC$K_DTYPE_D                ; for D_FLOAT divisor
            .BYTE       DSC$K_CLASS_S                ; to convert quadword
            .ADDRESS    D_FL_DIV                     ; time delta to D_FLOAT


	.psect	code	quad,rd,nowrt,exe
            .ENTRY      START,^M<>

            PUSHAQ      DES_D_DIV                   ; Set up divisor to convert
            PUSHAQ      DES_S_DIV                    ; quadword time deltas to
            CALLS       #2,G^LIB$CVT_DX_DX           ; double floating point
            BLBS        R0,INIT_TIME
            BRW         QUIT

INIT_TIME:  MOVAQ       TIME,R7                     ; Initialize timestamp and
            MOVAQ       TIME_DIFF,R8                 ; delta time ptr registers
            $FAO_S      CTRSTR = NOP_CTRL,-         ; Put lock operation count
                        OUTBUF = NOP_DESC,-          ; in result output string
                        OUTLEN = NOP_DESC,-
;;;                        OUTLEN = #5,-
                        P1 = #OP_COUNT
            BLBS        R0,NEW_LOCKS
            BRW         QUIT

;
; Loop to acquire new locks
;

NEW_LOCKS:  MOVL        #OP_COUNT,R6                ; Number of new locks
            MOVAB       NMSG1,R1                    ; Send message that new
            BSBW        PRINT_MSG                    ; acquired

            MOVAL       LK_ID_TBL,R9                ; R9 = ptr to lock id tbl
            SUBL3       #4,RESOURCE+4,R10           ; Get address of changing
            ADDW        RESOURCE,R10                 ; part of resource name
            $GETTIM_S   TIMADR = (R7)+              ; Get start timestamp

NEW_LOOP:   $ENQW_S     RESNAM = RESOURCE,-         ; Get a new lock
                        LKMODE = #LCK$K_CRMODE,-    
                        LKSB = LK_STS_BL
            BLBS        R0,LKSTS_NEW
            BRW         QUIT
LKSTS_NEW:  MOVZWL      LK_STS_BL,R0
            BLBS        R0,SAVE_LKID
            BRW         QUIT
SAVE_LKID:  MOVL        LK_STS_BL+4,(R9)+           ; Save new lock's lock id
            INCL        (R10)                       ; Increment resource name
            SOBGTR      R6,NEW_LOOP                 ; Continue to get new locks
            $GETTIM_S   TIMADR = (R7)               ; Get stop timestamp
;;            MOVQ        (R7)+,(R8)+                 ; Copy stop timestamp
            MOVL        (R7)+,(R8)+                 ; Copy stop timestamp
            MOVL        (R7)+,(R8)+                 ; Copy stop timestamp
;; gets rid of '%AMAC-I-QUADMEMREF, quadword memory references may be unaligned' errors
            MOVAB       NMSG2,R1                    ; Send message that new
            BSBW        PRINT_MSG                    ; acquired

;
; Loop to convert locks
;

            MOVL        #OP_COUNT,R6                ; Number of lock conversions
            MOVAL       LK_ID_TBL,R9                ; R9 = ptr to lock id tbl
            MOVAB       CMSG1,R1                    ; Send message that lock
            BSBW        PRINT_MSG                    ; progress
            $GETTIM_S   TIMADR = (R7)+              ; Get start timestamp
CNV_LOOP:   MOVL        (R9)+,LK_STS_BL+4           ; Get id of lock to convert
            MOVL        #0,LK_STS_BL                ; Clear status in LKSB
            $ENQW_S     FLAGS = #LCK$M_CONVERT,-    ; Convert mode of lock
                        LKMODE = #LCK$K_PRMODE,-
                        LKSB = LK_STS_BL
            BLBS        R0,LKSTS_CVT
            BRW         QUIT
LKSTS_CVT:  MOVZWL      LK_STS_BL,R0
            BLBS        R0,NEXT_CVT
            BRW         QUIT
NEXT_CVT:   SOBGTR      R6,CNV_LOOP                 ; Convert another lock
            $GETTIM_S   TIMADR = (R7)               ; Get stop timestamp
;;            MOVQ        (R7)+,(R8)+                 ; Copy stop timestamp
            MOVL        (R7)+,(R8)+                 ; Copy stop timestamp
            MOVL        (R7)+,(R8)+                 ; Copy stop timestamp
;; gets rid of '%AMAC-I-QUADMEMREF, quadword memory references may be unaligned' errors
            MOVAB       CMSG2,R1                    ; Send message that locks
            BSBW        PRINT_MSG

;
; Loop to release locks
;

            MOVL        #OP_COUNT,R6                ; Number of locks to release
            MOVAB       DMSG1,R1                    ; Send message that locks
            BSBW        PRINT_MSG
            MOVAL       LK_ID_TBL,R9                ; R9 = ptr to lock id tbl
            $GETTIM_S   TIMADR = (R7)+              ; Get start timestamp
DEQ_LOOP:   $DEQ_S      LKID = (R9)+                ; Release a lock
            BLBS        R0,NEXT_DEQ
            BRW         QUIT
NEXT_DEQ:   SOBGTR      R6,DEQ_LOOP                 ; Continue releasing locks
            $GETTIM_S   TIMADR = (R7)               ; Get stop timestamp
;;            MOVQ        (R7)+,(R8)+                 ; Copy stop timestamp
            MOVL        (R7)+,(R8)+                 ; Copy stop timestamp
            MOVL        (R7)+,(R8)+                 ; Copy stop timestamp
;; gets rid of '%AMAC-I-QUADMEMREF, quadword memory references may be unaligned' errors
            MOVAB       DMSG2,R1                    ; Send message that locks
            BSBW        PRINT_MSG

;
; Output results
;

OUT:        MOVAQ       TIME,R7                     ; Reinitialize timestamp and
            MOVAQ       TIME_DIFF,R8                 ; delta time ptr registers
            MOVL        #3,R6                       ; Three sets of results
            MOVAL       RMSG,R5                     ; Ptr to result msg header
LOOP_4:     SUBL        (R7),(R8)+                  ; Compute delta between
            SBWC        4(R7),(R8)+                  ; start and stop times
            MOVAQ       -8(R8),DES_S_DEL+4          ; Descriptor for delta
            PUSHAQ      DES_D_RNS                   ; Convert delta to ASCII
            PUSHAQ      DES_S_DEL                    ; text and store it in
            CALLS       #2,G^LIB$CVT_DX_DX           ; result string
            BLBS        R0,T_PER_LCK
            BRW         QUIT
T_PER_LCK:  PUSHAQ      DES_D_DEL                   ; Convert delta from
            PUSHAQ      DES_S_DEL                    ; quadword format
            CALLS       #2,G^LIB$CVT_DX_DX           ; to D_FLOAT format
            BLBS        R0,FL_DIV
            BRW         QUIT
FL_DIV:     DIVD        D_FL_DIV,D_FLOAT            ; Compute time per lck op,
            PUSHAQ      DES_USOP                     ; convert it to ASCII
            PUSHAQ      DES_D_DEL                    ; text, and store it
            CALLS       #2,G^LIB$CVT_DX_DX           ; in result string
            BLBS        R0,ASCII_TIM
            BRW         QUIT
ASCII_TIM:  $ASCTIM_S   TIMADR = (R7)+,-            ; Convert start and stop
                        TIMBUF = RSTR_DES            ; timestamps to ASCII
            $ASCTIM_S   TIMADR = (R7)+,-             ; text and store them
                        TIMBUF = RSTP_DES            ; in result string
            BLBS        R0,PRNT_RSLT
            BRW         QUIT

PRNT_RSLT:  MOVL        (R5)+,R1                    ; Print result string
            BSBW        PRINT_MSG                   ; of lock operations
            MOVAB       RESULT,R1
            MOVW        #RSLT_LEN,RESULT	    ; major kloodge 
            BSBW        PRINT_MSG
            ACBL        #1,#-1,R6,LOOP_4            ; Continue printing results

;
; Clean up and exit program
;

QUIT:       $EXIT_S     R0                          ; Exit program
	    ret
                                                   
;
; Subroutine to print message on terminal
;

PRINT_MSG:  
	.jsb_entry	input=<r1>
	    pushl	R1			
	    calls 	#1, g^lib$put_output

            BLBC        R0,QUIT2
            RSB
QUIT2:      $EXIT_S     R0                          ; Exit program
	    rsb

            .END        START
$EOD
$	endif
$! Check for the 
$	if f$search("locktime_3a.mar") .eqs. ""
$	then
$		create locktime_3a.mar
$DECK
	.library	/sys$library:lib/

;******************************************************************************
;                                                                             *
;  Program:    LOCKTIME_3                                                     *
;                                                                             *
;  Author:     Roy G. Davis                                                   *
;                                                                             *
;  Date:       August 12, 1993                                                *
;                                                                             *
;  Purpose:    This program is a companion to LOCKTIME_1.  It acquires locks  *
;              on all resource names used by LOCKTIME_1 so that the node on   *
;              which it runs masters the resource names used by LOCKTIME_1.   *
;                                                                             *
;  Summary:    After a small amount of initialization, this program acquires  *
;              locks on all resource names used by LOCKTIME_1.  This program  *
;              then pauses, waiting for the RETURN, ENTER, or SPACE key to be *
;              pressed.  When one of those is pressed, this program releases  *
;              all of its locks.                                              *
;                                                                             *
;  Parameters: This program should acquire locks on the same set of resource  *
;              names used by LOCKTIME_1.  Since this program uses the same    *
;              method as LOCKTIME_1 to generate resource names, the OP_COUNT  *
;              parameter in this program should be set to the same value as   *
;              in LOCKTIME_1.  See the program documentation at the beginning *
;              of LOCKTIME_1 for details on setting the OP_COUNT parameter.   *
;                                                                             *
;              The ENQ limit for the job from within which this program is    *
;              run should be set to at least the value of OP_COUNT plus the   *
;              maximum number of other locks held by the job while this       *
;              program is running.  If no other locks are held by the job,    *
;              then setting the ENQ limit to OP_COUNT+30 should generally     *
;              be sufficient.                                                 *
;                                                                             *
;              It is assumed that this program (LOCKTIME_3) is used to force  *
;              one node to master resource names used by LOCKTIME_1.  This    *
;              program is run on the node intended to master those resources, *
;              and LOCKTIME_1 is run on some other node.  To insure that      *
;              mastery of the resource names does not migrate from the node   *
;              on which LOCKTIME_3 runs to the node on which LOCKTIME_1 runs, *
;              the value of the SYSGEN parameter LOCKDIRWT should be greater  *
;              on LOCKTIME_3's node than on LOCKTIME_1's node.                *
;                                                                             *
;  Note:       Start LOCKTIME_3 on the node intended to master the resource   *
;              names used by LOCKTIME_1.  Then start LOCKTIME_1 on some other *
;              node after LOCKTIME_3 prompts for the RETURN, ENTER, or SPACE  *
;              key.  Finally, wait for LOCKTIME_1 to report results before    *
;              pressing ANY key on the terminal used to run LOCKTIME_3.       *
;                                                                             *
;  Assembly:   To assemble and link this program for execution, simply use    *
;              the following two VMS DCL commands:                            *
;                                                                             *
;                         MACRO name_of_this_program                          *
;                         LINK name_of_this_program                           *
;                                                                             *
;******************************************************************************
;
; Note: This slightly-modified version, known as LOCKTIME_3a, waits "forever"
; after acquiring the locks, instead of waiting for the user to press Space
; or Return or Enter.  This allows it to be run in batch in an automated
; fashion.  We simply delete the job which is running this image when we want
; to end this program.  (If run interactively, a user could use Control-Y
; and $STOP to end this program.)
;                     Keith Parris   3/2003
;

$LCKDEF
; User's ENQLM quota must be at least a bit bigger than the following number:
OP_COUNT = 5000

LF = 10
CR = 13
NULL = 0

	.psect	data	quad,rd,wrt,noexe
;
; Messages indicating when each loop begins and ends, and to request
; that RETURN, ENTER, or SPACE be pressed to release all locks.
;

NMSG1:      .ASCII      <CR><LF><LF>/Starting new lock acquisition/
            NMSG1_LEN = .-NMSG1
NMSG2:      .ASCII      <CR><LF>/New lock acquisition complete/
            NMSG2_LEN = .-NMSG2
DMSG1:      .ASCII      <CR><LF>/Starting lock dequeuing/
            DMSG1_LEN = .-DMSG1
DMSG2:      .ASCII      <CR><LF>/Lock dequeuing complete/<CR><LF>
            DMSG2_LEN = .-DMSG2
PRSKY:      .ASCII      <CR><LF><LF>/Press Control-Y to release all locks/-
                        <CR><LF>
            PRSKY_LEN = .-PRSKY


;TT_NAME:    .ASCID      /SYS$INPUT:/                ; Terminal I/O device
;TT_CHAN:    .BLKW       1                           ; I/O chan for terminal
;TT_IOSB:    .BLKQ       1                           ; IOSB for terminal $QIO
;TT_IN_BUF:  .BLKB       4                           ; Dummy input buffer
	.align	quad
msg_desc:
msg_len:	.BLKW	1
		.WORD	0
msg_addr:	.BLKL	1

LK_STS_BL:  .BLKQ       1                           ; LKSB for $ENQ
RESOURCE:   .ASCID      /ROYDAVIS/-
                        <NULL><NULL><NULL><NULL>    ; Resource name
LK_ID_TBL:  .BLKL       OP_COUNT                    ; Lock id storage

LONG_DELAY:	.F_FLOATING 99999	;Number of seconds to hold locks
					; (unless this process is killed first)

	.psect code	quad,rd,nowrt,exe

            .ENTRY      START,^M<>

;
; Initialization
;

;;ASSGN_CHN:  $ASSIGN_S   DEVNAM = TT_NAME,-          ; Open I/O channel to
;;                        CHAN = TT_CHAN               ; to SYS$OUTPUT
;;            BLBS        R0,NEW_LOCKS
;;            BRW         QUIT

;
; Loop to acquire new locks
;

NEW_LOCKS:  MOVL        #OP_COUNT,R6                ; Number of new locks
            MOVAB       NMSG1,R1                    ; Send message that new
            MOVL        #NMSG1_LEN,R2                ; locks are being
            BSBW        PRINT_MSG                    ; acquired
            MOVAL       LK_ID_TBL,R9                ; R9 = ptr to lock id tbl
            SUBL3       #4,RESOURCE+4,R10           ; Get address of changing
            ADDW        RESOURCE,R10                 ; part of resource name
NEW_LOOP:   $ENQW_S     RESNAM = RESOURCE,-         ; Get a new lock
                        LKMODE = #LCK$K_CRMODE,-    
                        LKSB = LK_STS_BL
            BLBS        R0,LKSTS_NEW
            BRW         QUIT
LKSTS_NEW:  MOVZWL      LK_STS_BL,R0
            BLBS        R0,SAVE_LKID
            BRW         QUIT
SAVE_LKID:  MOVL        LK_STS_BL+4,(R9)+           ; Save new lock's lock id
            INCL        (R10)                       ; Increment resource name
            SOBGTR      R6,NEW_LOOP                 ; Continue to get new locks
            MOVAB       NMSG2,R1                    ; Send message that new
            MOVL        #NMSG2_LEN,R2                ; locks have been
            BSBW        PRINT_MSG                    ; acquired

;
; Wait for Control-Y or for someone to kill the process
;

            MOVAB       PRSKY,R1                   ; Request Control-Y
            MOVL        #PRSKY_LEN,R2                ; be pressed to
            BSBW        PRINT_MSG                    ; release all locks
;; For this version of the program, which we'll run in batch, we'll just
;; wait until the job is deleted.  VMS will take care of releasing the locks.
;;            $QIOW_S     FUNC=#IO$_READVBLK,-       ; Wait for any key
;;                        CHAN=TT_CHAN,-
;;                        IOSB=TT_IOSB,-
;;                        P1=TT_IN_BUF,-
;;                        P2=#1
;;            BLBC        R0,QUIT
;;            MOVZWL      TT_IOSB,R0
;;            BLBC        R0,QUIT
	; Wait a very long time -- normally until the user forces an exit
            PUSHAL	LONG_DELAY	;Number of seconds to wait
            CALLS	#1,g^LIB$WAIT
            BLBC        R0,QUIT
;
; Loop to release locks
;

            MOVL        #OP_COUNT,R6                ; Number of locks to release
            MOVAB       DMSG1,R1                    ; Send message that locks
            MOVL        #DMSG1_LEN,R2                ; are being released
            BSBW        PRINT_MSG
            MOVAL       LK_ID_TBL,R9                ; R9 = ptr to lock id tbl
DEQ_LOOP:   $DEQ_S      LKID = (R9)+                ; Release a lock
            BLBS        R0,NEXT_DEQ
            BRW         QUIT
NEXT_DEQ:   SOBGTR      R6,DEQ_LOOP                 ; Continue releasing locks
            MOVAB       DMSG2,R1                    ; Send message that locks
            MOVL        #DMSG2_LEN,R2                ; have been released
            BSBW        PRINT_MSG

;
; Clean up and exit program
;

;;            $DASSGN_S   CHAN = TT_CHAN              ; Release I/O channel
;;            BLBC        R0,QUIT

QUIT:       $EXIT_S     R0                          ; Exit program
	    ret
                                                   
;
; Subroutine to print message on terminal
; R1 = address of message to print
; R2 = length of message

PRINT_MSG:  
	.jsb_entry	input=<r1,r2>
            MOVW	R2,msg_len	;Put length into descriptor
            MOVL	R1,msg_addr	;Put message address into descriptor
            PUSHAL	msg_desc
            CALLS	#1,g^LIB$PUT_OUTPUT
;;	    $QIOW_S     CHAN = TT_CHAN,-
;;                        FUNC = #IO$_WRITEVBLK,-
;;                        IOSB = TT_IOSB,-
;;                        P1 = (R1),-
;;                        P2 = R2
;;            BLBC        R0,QUIT2
;;            MOVZWL      TT_IOSB,R0
            BLBC        R0,QUIT2
            RSB
QUIT2:      $EXIT_S     R0                          ; Exit program
	    rsb

            .END        START
$EOD
$	endif
$	arch = f$getsyi("ARCH_NAME")
$	qual = "/migrate"
$	if arch .eqs. "VAX" then qual = ""
$! Link the Client program
$	macro'qual'/object=locktime_1_'arch'.obj/lis=locktime_1_'arch'.lis -
		locktime_1.mar
$	link/exe=locktime_1_'arch'.exe/nomap locktime_1_'arch'.obj
$	delete locktime_1_'arch'.obj;*
$! Link the Master program
$	macro'qual'/obj=locktime_3a_'arch'.obj/lis=locktime_3a_'arch'.lis -
		locktime_3a.mar
$	link/exe=locktime_3a_'arch'.exe/nomap locktime_3a_'arch'.obj
$	delete locktime_3a_'arch'.obj;*
$!
$	set default 'old_dir'
$	exit
