$!
$! SHOW_PATHS.COM
$!
$! Create a matrix showing SCS LAN connectivity among nodes in the cluster
$! (and to each of the other adapters on this same node).
$!
$!						V1.5 03/2003
$! Author: Keith Parris  parris@encompasserve.org or keithparris@yahoo.com
$! http://www.geocities.com/keithparris/ or http://encompasserve.org/~parris/
$!
$!	Modification history:
$!
$!	V1.0 March 2, 2001,	Written while helping SAIC, San Diego
$!	V1.1 March 12, 2001,	Modified at E*Trade to handle more nodes while
$!				still fitting output onto 80-column displays
$!	V1.1a Oct. 2002		Modified at HP-ETS 2002 to handle 7.3/7.3-1
$!				and their changes in SDA output formats
$!	V1.2 Nov. 2002		Incorporated SHOW_ALL_CHANNELS.COM
$!	V1.3 Dec. 2002		VAX support; more testing
$!	V1.4 Feb. 2003		Add support for Equivalent Channel Set info
$!				when running on V7.3 and above; in addition
$!				to connectivity, show whether or not a path
$!				is in the ECS, and whether it is Tight/Lossy
$!				in terms of packet loss history,
$!				Inferior/Peer/Superior in terms of
$!				of priority/packet-size, and Fast/Slow in
$!				terms of latency.  Removed device-type
$!				information, since we weren't displaying it
$!				anywhere anyway.
$!	V1.5 Mar. 2003		On pre-7.3 systems, indicate preferred path
$!				with a letter "P" instead of a plus-sign "+"
$!
$	if_debug = "!"	!Use "!" for normal operations, and "" for debugging
$	display_legend = "T"	!Use "T" to display legend, and "F" to suppress
$	if f$getsyi("VAXCLUSTER") .le. 0
$	then
$	    write sys$output "This system is not part of an OpenVMS Cluster."
$	    exit
$	endif
$	if (.not. f$getdvi("PEA0","EXISTS")) .or. -
	   (.not. f$getsyi("NISCS_LOAD_PEA0"))
$	then
$	    write sys$output "LAN SCS communications is disabled on this node."
$!	    write sys$output "NISCS_LOAD_PEA0 parameter value is ",-
$!_!			f$getsyi("NISCS_LOAD_PEA0"),"."
$		exit
$	endif
$	auth_privs = "," + f$getjpi("","AUTHPRIV") + ","
$	cur_privs = "," + f$getjpi("","CURPRIV") + ","
$! Check for OPER privilege (for SYSMAN) and turn it on if need be
$	OPER_TURNED_ON = "F"	!Assume process already had OPER privilege turned on
$	if f$locate(",OPER,",cur_privs) .eq. f$length(cur_privs)
$	then	!OPER privilege is not presently turned on
$		if f$locate(",OPER,",auth_privs) .ne. f$length(auth_privs) .or. -
		   f$locate(",SETPRV,",auth_privs) .ne. f$length(auth_privs)
$		then
$			set process/privilege=(OPER)
$			OPER_TURNED_ON = "T"
$		else
$			write sys$output -
		    "This procedure requires OPER privilege to run SYSMAN."
$			exit
$		endif
$	endif
$! Check for CMKRNL privilege rights
$	if f$locate(",CMKRNL,",cur_privs) .eq. f$length(cur_privs)
$	then	!CMKRNL privilege is not presently turned on
$		if f$locate(",CMKRNL,",auth_privs) .ne. f$length(auth_privs) .or. -
		   f$locate(",SETPRV,",auth_privs) .ne. f$length(auth_privs)
$		then	!CMKRNL is authorized
$		else
$			write sys$output -
		    "This procedure requires CMKRNL privilege to run SDA."
$			goto cleanup
$		endif
$	endif
$! Check for SYSPRV privilege rights (for SCACP)
$!!!	if V73 .and.
$	if f$locate(",SYSPRV,",cur_privs) .eq. f$length(cur_privs)
$	then	!SYSPRV privilege is not presently turned on
$		if f$locate(",SYSPRV,",auth_privs) .ne. f$length(auth_privs) .or. -
		   f$locate(",SETPRV,",auth_privs) .ne. f$length(auth_privs)
$		then	!SYSPRV is authorized
$		else
$			write sys$output -
		    "This procedure requires SYSPRV privilege to run SCACP."
$			goto cleanup
$		endif
$	endif
$! Check for LOG_IO privilege rights (for SCACP on VAX)
$!!!	if V73 .and.
$	if f$locate(",LOG_IO,",cur_privs) .eq. f$length(cur_privs)
$	then	!LOG_IO privilege is not presently turned on
$		if f$locate(",LOG_IO,",auth_privs) .ne. f$length(auth_privs) .or. -
		   f$locate(",SETPRV,",auth_privs) .ne. f$length(auth_privs)
$		then	!LOG_IO is authorized
$		else
$			write sys$output -
		    "This procedure requires LOG_IO privilege to run SCACP."
$			goto cleanup
$		endif
$	endif
$! Make temporary filenames unique by PID
$	pid = f$getjpi("","PID")
$	temp_file := show_paths_temp_'pid'.temp_file
$	delete := delete
$! See if this node is running V7.3 or later, with enhanced PEDRIVER support
$! and SCACP utility available
$	V73 = f$extract(1,99,f$getsyi("VERSION")) .ges. "7.3"
$! If this procedure is being called recursively, gather channel info -->
$	if p1 .eqs. "SHOW_ALL_CHANNELS" then goto 'p1'
$! Specify the maximum number of display columns for output, per page
$	MAX_COLUMNS = 79	!Can use up to 256 to keep more on a line
$	if p1 .nes. "" .and. f$integer(p1) .ge. 80 then -
		MAX_COLUMNS = f$integer(p1) - 1 
$	form_feed = " "
$	form_feed[0,7] = 12	!Form-feed character
$!
$! Gather SCS LAN-path info using a recursive call to this procedure (with the
$! SHOW_ALL_CHANNELS parameter) on each cluster node
$!
$	proc = f$environment("procedure")
$	proc_dir = f$parse(proc,,,"DEVICE") + f$parse(proc,,,"DIRECTORY")
$	close/nolog com
$	open/write com 'temp_file'_com
$	write com "$ mcr sysman set environment/cluster"
$	write com "set profile/priv=(cmkrnl,sysprv,log_io)"
$	write com "SET TIMEOUT 0:0:30"  !Give up after 30 seconds
$	write com "do @",proc," SHOW_ALL_CHANNELS"	!Call us recursively
$	write com "exit"
$	write com "$ exit"
$	close com
$	@'temp_file'_com/out='temp_file'_paths
$	delete 'temp_file'_com;*
$ 'if_debug' type/page/header 'temp_file'_paths	!!!
$! See if there were any error messages in the output
$	define/user sys$output _NLA0:
$	define/user sys$error _NLA0:
$	search/out=_NLA0: 'temp_file'_paths -
		"LAN SCS communications is disabled",-
		"This procedure requires",-
		error,"-f-","-e-","-w-"
$	if $severity .eq. 1	!1 means string found; 3 means none found
$	then	!Got some errors during data collection
$		write sys$output "Errors detected during data collection:"
$		write sys$output "---------------------------------------"
$		search 'temp_file'_paths ":["/match=nor
$		write sys$output "---------------------------------------"
$		dc_errors = "TRUE"
$	else
$		dc_errors = "FALSE"
$	endif
$! The SHOW_ALL_CHANNELS section below outputs a
$! "Channel data collection was successful on node xxxx" message
$! when it is able to gather path data on a node.
$! @@@@@ We should check each of these messages against our list of cluster
$! nodes and note any specific nodes where the data collection has failed for
$! any reason. @@@@@
$!
$! Sort nodenames and adapter names in alphabetical order for neatest output
$! (This sorts only the local node and local adapter; the remote nodes and
$! adapters are still unordered.  Note that if we detect failures during
$! data collection and use the remote nodename/adapter name to pick up some
$! of the missing data, the result is that the results may not be alphabetical
$! anymore.)
$	sort/key=(position:2,size:12)/duplicates -
		'temp_file'_paths 'temp_file'_paths_sorted
$	delete 'temp_file'_paths;*
$ 'if_debug' type/page/header 'temp_file'_paths_sorted
$! Read the sorted list of paths and create DCL symbols containing the
$! connectivity matrix info
$	node_list = ","	!List of nodes in the cluster
$	node_cnt = 0	!Count of nodes in the cluster
$	close/nolog paths
$	ecs_data_found = "F"	!Track whether V7.3 data was included, so
$!				!we can print a legend for that
$	open/read paths 'temp_file'_paths_sorted
$! Here's an example of what we're parsing (for pre-7.3 nodes):
$! [NCACAA]: EIA + EIB :[NCACAF]
$! [NCACAA]: EWC + EWC :[NCACAF]
$! [NCACAA]: EWC P EWD :[NCACAF]
$!               ^ ("+" indicates connectivity, "P" indicates preferred-path)
$!  2 <-- POSITION for SORT
$!  123456789.12 <-- SIZE for SORT
$! Note: For nodes at V7.3 or above, we'll see the ECS data added in the
$! middle in place of the plus sign, like this:
$! [NCACAA]: EIA N(L,P,S) EIB :[NCACAF]
$! [NCACAA]: EWC Y(T,P,F) EWC :[NCACAF]
$!Element: 0  1     2      3      4
$! where the ECS data is for the path from the viewpoint of the node on the left.
$ loop:
$	read/end=eof paths rec
$	if f$extract(0,1,rec) .nes. "[" then goto loop
$	loc_node = f$element(0," ",rec) - "[" - "]" - ":"
$	loc_dev_name = f$element(1," ",rec)
$	path_ecs = f$element(2," ",rec)		!"+" or "P" or ECS data from SCACP
$	rmt_dev_name = f$element(3," ",rec)
$	rmt_node = f$element(4," ",rec) - "[" - "]" - ":"
$! See if the node is in our list
$	if f$locate(("," + loc_node + ","),node_list) .eq. f$length(node_list)
$	then
$		node_list = node_list + loc_node + ","
$		node_cnt = node_cnt + 1
$	endif
$! Record the path between adapters for the connectivity matrix we're making
$	PATH_'loc_node'_'loc_dev_name'_'rmt_node'_'rmt_dev_name' = path_ECS
$! Record the path in the reverse direction also, if it's not already noted.
$! Note: Because we don't know yet if the other node runs 7.3 or if it has ECS
$! data, we'll just record path existence with a plus sign, instead of using any
$! ECS data we might have in the forward path direction.  Unless we are
$! unable to gather data on the remote node itself, if it has ECS data, that
$! will override the simple "x" path reachability indication we insert here.
$	if f$type(PATH_'rmt_node'_'rmt_dev_name'_'loc_node'_'loc_dev_name') -
		.eqs. "" then -
		PATH_'rmt_node'_'rmt_dev_name'_'loc_node'_'loc_dev_name' = "x"
$! Now record the adapter, if we don't already have a record of it in the
$! list for the associated node.
$! First, see if the node has an adapter list yet.  If not, create one.
$	if f$type(ADP_'loc_node') .eqs. ""
$	then	!Adapter list needs to be created for this node
$		ADP_'loc_node' = ","
$		ADP_'loc_node'_CNT = 0
$	endif
$! See if the local adapter is in our adapter list for the node
$	if f$locate(("," + loc_dev_name + ","),ADP_'loc_node') -
		.eq. f$length(ADP_'loc_node')
$	then
$		ADP_'loc_node' = ADP_'loc_node' + loc_dev_name + ","
$		ADP_'loc_node'_CNT = ADP_'loc_node'_CNT + 1
$	endif
$	goto loop
$ eof:
$	close paths
$! If we had data-collection errors, we may be missing data from some nodes.
$! In that case, we'll make another pass through the list, picking up the
$! remote node names (and any adapters attached to them that we've detected).
$! by looking at each indicated path from the opposite end -- that
$! of the remote node and remote adapter.  We'll still be missing data on
$! connections between the set of nodes with data-collection errors, but at
$! least we'll display all the remote nodes and adapters we're aware of.  Any
$! paths where we're missing connectivity data, we'll be putting a "?" in
$! instead of a "+", space (indicating _no_ connectivity), or ECS data.
$!
$! (We do this scan as a second pass because if we try to pick up the remote
$! nodenames and adapters in the initial pass, it greatly messes up our
$! alphabetical ordering of nodenames and adapter names.  By adding missing
$! nodes and adapters at the end, less non-alphabetic ordering of node and
$! adapter names results.)
$!!	if .not. dc_errors then goto end_pass2	!Skip if we got all the data
$!!! We'll go ahead and do the second pass all the time, just in case an
$!!! error in data collection slips past our detection code.
$	dc_err_nodes = ","
$	open/read paths 'temp_file'_paths_sorted
$! As a reminder, here's an example of what we're parsing:
$! [NCACAA]: EIA <EI_82559> + <EI_82559> EIB :[NCACAF]
$! [NCACAA]: EIA <EI_82559> P <EI_82559> EIC :[NCACAF]
$! [NCACAA]: EWC <EW_DE500> Y(T,P,F) <EW_DE500> EWC :[NCACAF]
$ loop2:
$	read/end=eof2 paths rec
$	if f$extract(0,1,rec) .nes. "[" then goto loop2
$	loc_node = f$element(0," ",rec) - "[" - "]" - ":"
$	loc_dev_name = f$element(1," ",rec)
$!!!	path_ecs = f$element(2," ",rec)		!"+" or "P" or ECS data from SCACP
$	rmt_dev_name = f$element(3," ",rec)
$	rmt_node = f$element(4," ",rec) - "[" - "]" - ":"
$! See if the node is in our list
$	if f$locate(("," + rmt_node + ","),node_list) .eq. f$length(node_list)
$	then	!Not there -- add it
$		node_list = node_list + rmt_node + ","
$		node_cnt = node_cnt + 1
$! Also add it to our list of nodes for which data is apparently missing
$		dc_err_nodes = dc_err_nodes + rmt_node + ","
$	endif
$! Now record the adapter, if we don't already have a record of it in the
$! list for the associated node.
$! First, see if the node has an adapter list yet.  If not, create one.
$	if f$type(ADP_'rmt_node') .eqs. ""
$	then	!Adapter list needs to be created for this node
$		ADP_'rmt_node' = ","
$		ADP_'rmt_node'_CNT = 0
$	endif
$! See if the remote adapter is in our adapter list for the remote node
$	if f$locate(("," + rmt_dev_name + ","),ADP_'rmt_node') -
		.eq. f$length(ADP_'rmt_node')
$	then
$		ADP_'rmt_node' = ADP_'rmt_node' + rmt_dev_name + ","
$		ADP_'rmt_node'_CNT = ADP_'rmt_node'_CNT + 1
$	endif
$	goto loop2
$ eof2:
$	close paths
$!! end_pass2:
$	delete 'temp_file'_paths_sorted;*
$ 'if_debug' show symbol node_cnt	!!!
$ 'if_debug' show symbol node_list	!!!
$ 'if_debug' show symbol dc_err_nodes	!!!
$ if dc_err_nodes .nes. "," then -
	write sys$output "*** Data appears to be missing for node(s): ",-
		f$extract(1,f$length(dc_err_nodes)-2,dc_err_nodes)," ***"
$ 'if_debug' show symbol ADP_*	!!!
$ 'if_debug' show symbol PATH_*	!!!
$!
$! Now we know about all the adapters & nodes, and the connectivity between
$! them.
$!
$! Create a matrix of the following form:
$!            Node_1      Node_2          Node_3 Node_4
$!	      Ad1 Ad2 Ad3 Ad1 Ad2 Ad3 Ad4 Ad1    Ad1 Ad2
$! Node_1 Ad1      +   +       +           +          +
$!        Ad2  +           +           +          +
$!        Ad3  +               +       +          +
$! Node_2 Ad1  +       +           +       +          +
$!  etc., where "+" represents SCS connectivity exists between those adapters,
$!  "P" indicates a preferred path between nodes, a blank 
$!  represents no SCS connectivity, and a question mark "?" means
$!  we don't know because we're missing data from one or more nodes,
$!  and "x" indicates we know we have connectivity in the reverse direction
$!  of this path but have no data about this path in the forward direction,
$!  and a construct of the form Y(T,P,F) represents the Equivalent Channel
$!  Set (ECS) data from SCACP, available in VMS 7.3 and above:
$!	Y/N   Is path in the Equivalent Channel Set (ECS): Yes or No
$!	T/L   Packet loss history: Tight or Lossy
$!	I/P/S Priority and/or Maximum Packet Size: Inferior/Peer/Superior
$!	F/S   Latency: Fast or Slow
$!
$! To fit this info into 3 columns (the width of an adapter name) use:	
$!	If path is in the ECS (Y), use uppercase letters and show the 3
$!	e.g. "TPF" letters.  If path is NOT in the ECS (N), use lowercase
$!	letters and show the 3 e.g. "lis" letters.
$!	Consider using ">" for Superior, "=" for Peer, and "<" for Inferior (?)
$!	Consider using colors (i.e. red indicates paths not in the ECS)
$!	or reverse-video
$!---------
$! Basic algorithm is:
$!   For each local node...
$!   For each adapter on that node, do a line of output:
$!   For each node...
$!   For each adapter on that node...
$!   Show the connectivity between that pair of adapters, if present
$!
$! Width of left-hand fixed area is <6 bytes for nodename> + 1 blank +
$!         <3 bytes for device name>
$! Width of a node's column is MIN(7,4x<#adapters>) spaces
$! Width of an adapter's column is 4, or 7 if adapter count for that node is 1.
$! 
$! If we have too many nodes/adapters to fit in the width of one page (which is
$! specified by symbol MAX_COLUMNS), we divide the output into multiple pages.
$	LAST_I = 0	!Start with first node (1) for first page
$! We loop here for each page of output:
$ table_loop:	!Output the main table of output (a piece at a time if need be)
$	FIRST_I = LAST_I + 1	!Start with next node as we begin a new page
$! Write the nodename and adapter name column headings at the top
$	gosub print_heading
$!
$! Begin generating the main area of the matrix a line at a time.
$!
$! Process each and every node in turn:
$	i = 1	!I is the index of the node at the left-hand side, for this row
$ loop_loc_node:
$	loc_node = f$element(i,",",node_list)
$	node_field = loc_node	!Set up to put nodename into the left-hand column
$! Now, for each adapter on the local node:
$	j = 1	!J is the index into the adapters on node 'i'
$ loop_loc_adp:
$	loc_dev_name = f$element(j,",",ADP_'loc_node')
$	line = f$fao("!6AS !3AS",node_field,loc_dev_name)
$	node_field = " "	!Blank out nodename for succeeding adapters
$!				!on same node
$! Now, for each remote node: (where each "remote" node has a column heading)
$	k = FIRST_I	!K is the index into the column-heading nodes
$ loop_rmt_node:
$	rmt_node = f$element(k,",",node_list)
$! Now, for each adapter on the remote node:
$	m = 1		!M is the index into the adapters on node 'k'
$ loop_rmt_adp:
$	rmt_dev_name = f$element(m,",",ADP_'rmt_node')
$	t = f$type(PATH_'loc_node'_'loc_dev_name'_'rmt_node'_'rmt_dev_name')
$	if t .nes. "" then t = PATH_'loc_node'_'loc_dev_name'_'rmt_node'_'rmt_dev_name'
$	if f$locate(",",t) .ne. f$length(t)
$	then	!ECS data present; 
$! To fit the ECS info into 3 columns (the width of an adapter name),
$! If path is in the ECS (Y), use uppercase letters and show the 3
$! e.g. "TPF" letters.  If path is NOT in the ECS (N), use lowercase
$! letters and show the 3 e.g. "lis" letters.
$!	Consider using ">" for Superior, "=" for Peer, and "<" for Inferior (?)
$!	Consider using some other better abbreviations for Tight/Lossy packet
$!	loss history and Fast/Slow latency measurement
$		letters = f$extract(2,5,t) - "," - ","
$		if f$extract(0,1,t) .nes. "Y" then -
			letters = f$edit(letters,"LOWERCASE")
$		t = letters
$		ecs_data_found = "T"	!Now we need to print a legend
$	endif
$	if loc_node .eqs. rmt_node .and. loc_dev_name .eqs. rmt_dev_name then -
		t = "\"	!Adapter to itself -- Diagonal centerline of the matrix
$! If both the local and remote nodes are on our list of nodes for which data
$! is missing, and thus we don't know if there is connectivity betwen them,
$! flag that fact by putting a question mark at the junction.
$	if (t .eqs. "") .and. -
    (f$locate(","+loc_node+",",dc_err_nodes) .ne. f$length(dc_err_nodes) .and. -
     f$locate(","+rmt_node+",",dc_err_nodes) .ne. f$length(dc_err_nodes))
$	then
$		t = "?"
$!!!		PATH_'loc_node'_'loc_dev_name'_'rmt_node'_'rmt_dev_name' = "?"
$	endif
$	if f$length(t) .ge. 2
$	then	!Width of 3
$		line = line + f$fao(" !AS",t)
$	else	!Width of 1 -- put a blank on either side
$		line = line + f$fao("  !AS ",t)
$	endif
$! Repeat for each remote adapter
$	m = m + 1
$	if m .le. ADP_'rmt_node'_CNT then goto loop_rmt_adp
$! A little extra column fill for 'adapter-challenged' nodes...
$	if ADP_'rmt_node'_CNT .eq. 1 then line = line + "   "
$! Repeat for each remote node
$	k = k + 1
$	if k .le. LAST_I then goto loop_rmt_node
$! Now we have all the info on the remote nodes and adapters.  Write the line.
$	write sys$output line
$! Repeat for each local adapter
$	j = j + 1
$	if j .le. ADP_'loc_node'_CNT then goto loop_loc_adp
$! Repeat for each local node
$	i = i + 1
$! See if there are more nodes to process
$	if i .le. node_cnt then goto loop_loc_node
$! Now see if we've handled all the nodes, or if we need to do another page
$	if LAST_I .lt. node_cnt
$	then
$!!!		write sys$output ""	!Blank line between pages (or FF?)
$		write sys$output form_feed	!Form-feed between pages
$		goto table_loop
$	endif
$	if .not. display_legend then goto after_legend
$	write sys$output ""	!Blank link after table
$	write sys$output " Legend:"
$	write sys$output "      +    Path is present"
$	write sys$output "      P    Preferred Path"
$	write sys$output "   <blank> No path present
$	write sys$output "      ?    Data missing: path status is indeterminate"
$	write sys$output "      x    Path appears to be present in opposite direction only"
$	write sys$output "      \    Path to self"
$	if ecs_data_found
$	then
$!		write sys$output -
$!_!"  -- Equivalent Channel Set (ECS) data legend --"
$!		write sys$output -
$!_!"                Letter codes: 123"
$!		write sys$output "     123   ECS data, as follows:
$!		write sys$output -
$!_!"  1: Packet loss history:   T=Tight, L=Lossy"
$!		write sys$output -
$!_!"  2: Priority/Packet Size:  I=Inferior, P=Peer, S=Superior"
$!		write sys$output -
$!_!"  3: Latency:               F=Fast, S=Slow"
$!		write sys$output -
$!_!"(Capital letters denote paths in the ECS; lowercase for paths NOT in the ECS.)"
$!		write sys$output "     xyz   ECS data, as follows:
$!!		write sys$output "     ^^^"
$!		write sys$output "     |||"
$!		write sys$output "     ||+-- Latency:               F=Fast, S=Slow"
$!		write sys$output "     ||"
$!		write sys$output "     |+--- Priority/Packet Size:  I=Inferior, P=Peer, S=Superior"
$!		write sys$output "     |"
$!		write sys$output "     +---- Packet loss history:   T=Tight, L=Lossy"
$!!		write sys$output ""
$!		write sys$output "     (Capital letters denote paths in the ECS; lowercase paths NOT in the ECS.)"
$!		write sys$output -
$!_!"     xyz   ECS data, as follows:                                   x y z"
$		write sys$output -
"     xyz   Equivalent Channel Set (ECS) data, as follows:          x y z"
$		write sys$output -
"                                                                   | | |"
$		write sys$output -
"          Packet loss history:   T=Tight, L=Lossy -----------------+ | |"
$		write sys$output -
"          Priority/Packet Size:  I=Inferior, P=Peer, S=Superior -----+ |"
$		write sys$output -
"          Latency:               F=Fast, S=Slow -----------------------+"
$		write sys$output -
"    (Capital letters denote paths in the ECS; lowercase, paths NOT in the ECS.)"
$	endif
$ after_legend:
$!
$! All done.  Clean up and exit.
$!
$ cleanup:
$! If we turned on OPER priv, turn it off again
$	if OPER_TURNED_ON then set process/privilege=(noOPER)
$	exit
$!
$! Routine to create and print headings
$!
$! Input:  Symbol FIRST_I is index of 1st node whose column is handled this pass
$! Output: Symbol LAST_I is index of last node whose column is handled this pass
$!
$ print_heading:
$	i = FIRST_I
$!		    1234567890	!10 chars: 6 node, 1 blank, 3 adapter
$!                  nodexx adp
$!!	hd1_line = "          "	!Blanks: nodename and adapter name
$	hd1_line = "From | To:"
$!!	hd2_line = hd1_line	!Same prefix width on next line
$	hd2_line = "---- v -->"
$ loop_hd1:
$	node = f$element(i,",",node_list)
$	width = 7	!Width of the vertical column for this node (all
$!			!adapters included)
$	t = ADP_'node'_CNT * 4	!If more than 2 adapters, need more width
$	if t .gt. width then width = t
$! See if things would still fit in one page-width if we added this node's info
$	if ((f$length(hd1_line) + width) .gt. max_columns) .and. -
	    (i .gt. FIRST_I)	!In case one node has so many adapters it
$!				!doesn't fit within MAX_COLUMNS -- do it anyway
$	then	!Used up all available column space; will have to do another
$!		!portion of the table in another pass
$		goto hd_end	!Stop before handling this next node
$	endif
$	t = width - 1
$	hd1_line = hd1_line + f$fao(" !''t'AS",node)
$	j = 1	!J is index into list of adapters for node at index I
$ loop_hd2:
$	adp = f$element(j,",",ADP_'node')
$	j = j + 1
$	hd2_line = hd2_line + " " + adp
$	if j .le. ADP_'node'_CNT then goto loop_hd2
$	if ADP_'node'_CNT .eq. 1 then hd2_line = hd2_line + "   " !Extra 3 blanks
$	LAST_I = i	!Remember the last node we were able to fit in
$	i = i + 1
$	if i .le. node_cnt then goto loop_hd1
$ hd_end:
$	write sys$output hd1_line	!Heading, line 1
$	write sys$output hd2_line	!Heading, line 2
$	return	!End of subroutine
$!
$ SHOW_ALL_CHANNELS:
$ if_standalone = "!"	!"" for use as a standalone procedure; "!" for use
$!			!within SHOW_PATHS.COM or SHOW_SEGMENTS.COM.
$!
$! SHOW_ALL_CHANNELS_ECS.COM
$!
$! Show all the active LAN channels (paths) seen by PEDRIVER to each remote node
$!
$! Based on SHOW_PREF_PATH.COM, SHOW_ALL_PATHS.COM
$!						V1.0 3/2001
$! Author: Keith Parris  parris@encompasserve.org or keithparris@yahoo.com
$! http://www.geocities.com/keithparris/ or http://encompasserve.org/~parris/
$! Modified for 7.3-1    KP 10/8/2002
$!	Output format changed for SDA> SHOW PORTS/VC/NODE=xxx
$! Improved		 KP 12/4/2002
$!	VAX SDA apparently doesn't support /NOINDEX qualifer to SET OUTPUT command
$!	Modified to also show paths between different LAN adapters on the same
$!	node, to fix problems when used for SHOW_SEGMENTS.COM, with its need to
$!	map adapters to network segments.
$!	Improved error-checking and privilege handling
$! Improved		KP 2/13/2003
$!	Added support for SCACP Equivalent Channel Set (ECS) info
$!	available on OpenVMS V7.3 and above via the new SCACP utility
$!
$	if_debug = "!"	!Use "!" for normal operations, and "" for debugging
$! Check to see if SCS communications over the LAN is even enabled on this node.
$! If not, SDA> SHOW PORT will not define the PE_PDT symbol and we can't
$! gather any data.
$	if (.not. f$getdvi("PEA0","EXISTS")) .or. -
	   (.not. f$getsyi("NISCS_LOAD_PEA0"))
$	then
$	    write sys$output "LAN SCS communications is disabled on node ",-
			f$getsyi("NODENAME"),"."
$!	    write sys$output "NISCS_LOAD_PEA0 parameter value is ",-
$!_!			f$getsyi("NISCS_LOAD_PEA0"),"."
$		exit
$	endif
$!
$	data_found = "F"	!Assume we failed until proven otherwise
$! Check for the privileges required to run SDA and/or SCACP
$	auth_privs = "," + f$getjpi("","AUTHPRIV") + ","
$	cur_privs = "," + f$getjpi("","CURPRIV") + ","
$! Remember whether or not we had to elevate privileges
$	ELEVATED_PRIVS = "F"	!Assume process already had CMKRNL privilege turned on
$	remove_privs = ""
$! Check for CMKRNL privilege and turn it on if need be
$	if (.not. V73)
$	then	!We'll have to use SDA -- SCACP isn't available yet
$	    if f$locate(",CMKRNL,",cur_privs) .eq. f$length(cur_privs)
$	    then	!CMKRNL privilege is not presently turned on
$		if f$locate(",CMKRNL,",auth_privs) .ne. f$length(auth_privs) .or. -
		   f$locate(",SETPRV,",auth_privs) .ne. f$length(auth_privs)
$		then
$			set process/privilege=(CMKRNL)
$			remove_privs = remove_privs + ",noCMKRNL"
$			ELEVATED_PRIVS = "T"
$		else
$			write sys$output -
		    "This procedure requires CMKRNL privilege to run SDA."
$			exit
$		endif
$	    endif
$! Check for READALL privilege and turn it on if possible
$	    if f$locate(",READALL,",cur_privs) .eq. f$length(cur_privs)
$	    then	!READALL privilege is not presently turned on
$		if f$locate(",READALL,",auth_privs) .ne. f$length(auth_privs) .or. -
		   f$locate(",SETPRV,",auth_privs) .ne. f$length(auth_privs)
$		then
$			set process/privilege=(READALL)
$			remove_privs = remove_privs + ",noREADALL"
$			ELEVATED_PRIVS = "T"
$!		else
$! Most systems seem to run SDA with only CMKRNL -- I've only hit a few
$! that apparently have protection problems or something and require READALL.
$! If the user doesn't have READALL, we'll just try SDA anyway without it.
$!			write sys$output -
$!_!		    "This procedure requires READALL privilege to run SDA."
$!			exit
$		endif
$	    endif
$	endif
$! Check for SYSPRV privilege and turn it on if need be
$	if V73
$	then	!We'll use SCACP instead of SDA
$	    if f$locate(",SYSPRV,",cur_privs) .eq. f$length(cur_privs)
$	    then	!,SYSPRV privilege is not presently turned on
$		if f$locate(",SYSPRV,",auth_privs) .ne. f$length(auth_privs) .or. -
		   f$locate(",SETPRV,",auth_privs) .ne. f$length(auth_privs)
$		then
$			set process/privilege=(SYSPRV)
$			remove_privs = remove_privs + ",noSYSPRV"
$			ELEVATED_PRIVS = "T"
$		else
$			write sys$output -
		    "This procedure requires SYSPRV privilege to run SCACP."
$			exit
$		endif
$	    endif
$! Check for LOG_IO privilege and turn it on if possible
$	    if f$locate(",LOG_IO,",cur_privs) .eq. f$length(cur_privs)
$	    then	!LOG_IO privilege is not presently turned on
$		if f$locate(",LOG_IO,",auth_privs) .ne. f$length(auth_privs) .or. -
		   f$locate(",SETPRV,",auth_privs) .ne. f$length(auth_privs)
$		then
$			set process/privilege=(LOG_IO)
$			remove_privs = remove_privs + ",noLOG_IO"
$			ELEVATED_PRIVS = "T"
$!		else
$! Most systems seem to run SCACP with only SYSPRV -- I've only hit a few
$! that apparently require LOG_IO.
$! If the user doesn't have LOG_IO, we'll just try SCACP anyway without it.
$!			write sys$output -
$!_!		    "This procedure requires LOG_IO privilege to run SCACP."
$!			exit
$		endif
$	    endif
$	endif
$	remove_privs = remove_privs - ","	!Remove leading comma
$! Make temporary filenames unique by PID
$!In captive mode, use temporary filenames from main procedure
$ 'if_standalone' pid = f$getjpi("","PID")
$ 'if_standalone' temp_file := show_all_channels_temp_'pid'.temp_file
$	delete := delete
$! Find out what node we're running on
$	this_node = f$getsyi("NODENAME")	!This is where we're running
$ 'if_debug' show symbol this_node
$! See if this node is running V7.3 or later, with enhanced PEDRIVER support
$! and SCACP utility available
$ 'if_standalone' V73 = f$extract(1,99,f$getsyi("VERSION")) .ges. "7.3"
$! If we're running V7.3 or later, use SCACP to gather path info.  If
$! we're running an earlier version, use SDA.
$	if V73 then goto USE_SCACP
$! Pre-7.3.  Use SDA to gather path info.
$!
$! First, find out the names of all the nodes in the cluster.  We'll need
$! that to create 'SDA> SHOW PORT/NODE=xxx' command lines, one per node.
$	nodecount = 0
$	context = ""
$ loop:
$	csid = f$csid(context)	!Get the CSID of the next cluster member
$	if csid .nes. ""
$	then
$	    nodecount = nodecount + 1	!1 more node
$	    nodename_'nodecount' = f$getsyi("NODENAME",,csid)
$ 'if_debug' show symbol nodename_'nodecount'
$	    goto loop
$	endif
$ 'if_debug' show symbol nodecount
$! Next, use SDA to find out the channel addresses of all the LAN paths
$! to each node
$	close/nolog com
$	open/write com 'temp_file'_com
$	write com "$ DEFINE/USER SYS$OUTPUT NLA0:"
$	write com "$ DEFINE/USER SYS$ERROR NLA0:"
$	write com "$ ANALYZE/SYSTEM"
$	out_qual = "/NOINDEX"
$	arch = f$getsyi("ARCH_NAME")
$	if arch .eqs. "VAX" then out_qual = ""
$	write com "SET OUTPUT''out_qual' _NLA0:"
$	write com "SHOW PORTS"	!To set symbols for use later
$	write com "SET OUTPUT''out_qual' ",temp_file,"_LIST"
$	i = 0
$ loop2:
$	i = i + 1
$!(Don't bother to look up a path to this node itself)
$!! Actually, we DO need to look up paths to this node itself.  We'll ignore
$!! the loopback path and path from an adapter to itself, but we will need to
$!! know (for SHOW_PATHS.COM and SHOW_SEGMENTS.COM) whenever there is a path
$!! between two different LAN adapters on the same node, because that indicates
$!! that they are connected to the same VLAN or extended LAN or network segment,
$!! however you want to phrase it.
$!!	if nodename_'i' .nes. this_node then-
$		write com "SHOW PORT/ADDRESS=PE_PDT/VC/NODE=",nodename_'i'
$	if i .lt. nodecount then goto loop2
$	write com "EXIT"
$	write com "$ EXIT"
$	close com
$ 'if_debug' type/page/header 'temp_file'_com
$	@'temp_file'_com
$! copy 'temp_file'_com chan.com
$	delete 'temp_file'_com;*	!!!
$	if f$search("''temp_file'_list") .eqs. ""
$	then	!SDA must have failed to write the file
$		write sys$output -
    "*** Error: SDA> SHOW PORT output failure on node ",this_node," ***"
$		goto cleanup
$	endif
$ 'if_debug' type/page/header 'temp_file'_list
$!Suppress "%SEARCH-I-NOMATCHES, no strings matched
$! and      %SEARCH-I-NULLFILE, file xxxxxx contains no records
$	define/user sys$output _NLA0:
$	define/user sys$error _NLA0:
$	search 'temp_file'_list "Preferred","Active","path,open"-
		/out='temp_file'_channels
$	search_severity = $severity
$ 'if_debug' show symbol search_severity
$	delete 'temp_file'_list;*	!!!
$	if search_severity .eq. 3	!No channel data found
$	then	!SDA must have failed to write anything useful to the file
$		write sys$output -
    "*** Error: SDA> SHOW PORT output empty on node ",this_node," ***"
$		goto cleanup
$	endif
$ 'if_debug' type/page/header 'temp_file'_channels
$	close/nolog chan
$! Now use SDA again to find out the "bus" (LAN adapter) which is currently
$! used in the path to each node
$	open/write com 'temp_file'_com2
$	write com "$ DEFINE/USER SYS$OUTPUT _NLA0:"
$	write com "$ DEFINE/USER SYS$ERROR _NLA0:"
$	write com "$ ANALYZE/SYSTEM"
$	write com "SET OUTPUT''out_qual' ",temp_file,"_LIST2"
$	open/read chan 'temp_file'_channels
$!
$! Pick up addresses of data structures for all channels that are working.
$! Here's what we're parsing:
$!
$!Address     Type    Xmt Time Size Preferred    Best        Last State Change
$!--------  --------- -------- ---- ---------  --------   -----------------------
$!81764F00  Preferred 00C59486 1412       289         4   28-FEB-2001 15:25:16.19
$!81764E00  Active    00C59486 1412         3       201   28-FEB-2001 15:25:15.83
$!
$! For 7.3-1:
$!Address   LCL/RMT   State      ECS   Size  AvgRTT(us)  Last State Change
$!--------  -------  --------   -----  ----  ----------  -----------------------
$!812D8540  EWA/EWA  path,open  T/P/S  1426     24220.9   7-OCT-2002 21:36:57.12
$!812B9340  EWB/EWB  path,open  T/P/F  1426       244.1   7-OCT-2002 21:24:40.58
$ loop3:
$	read/end=eof3 chan rec
$	rec = f$edit(rec,"COMPRESS,TRIM")
$!Try the 7.3-1 variant first
$	state = f$element(2," ",rec)
$	if state .eqs. "path,open" then goto got_chan
$	path_type = f$element(1," ",rec)
$	if path_type .nes. "Preferred" .and. path_type .nes. "Active" then -
		goto loop3
$ got_chan:
$!! 'if_debug' show symbol rec
$	chan_addr = f$element(0," ",rec)
$!! We're looking for an 8-digit hexadecimal address
$!!	if f$length(chan_addr) .ne. 8 then goto loop3
$	write com "SHOW PORT/CHANNEL=",chan_addr," !"	!,path_type
$	if path_type .eqs. "Preferred" then -
$		preferred_'chan_addr' = "T"	!Remember paths was Preferred
$	goto loop3
$ eof3:
$	close chan
$	delete 'temp_file'_channels;*	!!!
$	close com
$ 'if_debug' type/page/header 'temp_file'_com2
$	@'temp_file'_com2
$	delete 'temp_file'_com2;*	!!!
$	if f$search("''temp_file'_list2") .eqs. ""
$	then	!SDA must have failed to write the file
$		write sys$output -
    "*** Error: SDA> SHOW CHANNEL output failure on node ",this_node," ***"
$		goto cleanup
$	endif
$ 'if_debug' type/page/header 'temp_file'_list2
$!
$! Show the user what we found
$!
$!Suppress "%SEARCH-I-NOMATCHES, no strings matched
$! and      %SEARCH-I-NULLFILE, file xxxxxx contains no records
$	define/user sys$output _NLA0:
$	define/user sys$error _NLA0:
$	search 'temp_file'_list2 -
		"PEDRIVER Channel","Lcl Device:","Rmt Device:"-
		/out='temp_file'_list3
$	search_severity = $severity
$ 'if_debug' show symbol search_severity
$	delete 'temp_file'_list2;*	!!!
$	if search_severity .eq. 3
$	then	!SDA must have failed to write anything useful to the file
$		write sys$output -
    "*** Error: SDA> SHOW CHANNEL output empty on node ",this_node," ***"
$		goto cleanup
$	endif
$ 'if_debug' type/page/header 'temp_file'_list3
$	close/nolog tmp
$	open/read tmp  'temp_file'_list3
$! Here's an example of what we're parsing in loop 4:
$! -- PEDRIVER Channel (CH:81A71D40) for Virtual Circuit (VC:81A5FA40) NCACAB --
$!BUS: 819CB580  (EWB)  Lcl Device: EW_DEGPA  Lcl LAN Address: 00-60-6D-21-32-B4
$!Rmt Name: EWB         Rmt Device: EW_DEGPA  Rmt LAN Address: 00-60-6D-21-32-C0
$! For 7.3-1:
$! -- PEDRIVER Channel (CH:81438D00) for Virtual Circuit (VC:81048A80) HOWS0F --
$!State: 0004 open                Status: 6B path,open,rhwaval,tight,fast
$!                                ECS Status: Tight,Fast
$!BUS: 811A2600 (EWA)  Lcl Device: EW_DE500  Lcl LAN Address: 00-10-64-30-F1-B2
$!Rmt BUS Name:  EWA   Rmt Device: EW_DE500  Rmt LAN Address: 00-10-64-31-16-0C
$ loop4:
$	read/end=eof4 tmp rec
$	crec = f$edit(rec,"TRIM,COMPRESS")
$	if f$extract(0,19,crec) .eqs. "-- PEDRIVER Channel"
$	then
$		rmt_node = f$element(8," ",crec)	!Remote node's name
$		goto loop4
$	endif
$	if f$extract(0,4,rec) .eqs. "BUS:"
$	then
$		loc_dev_name = f$element(2," ",crec) - "(" - ")"	!e.g. EWA
$		chan_addr = f$element(1," ",crec)	!Channel address
$!!!		loc_dev_type = f$element(5," ",crec)		!e.g. EW_DEGPA
$!!!		loc_dev_addr = f$elmenet(9," ",crec)	!MAC address
$		goto loop4
$	endif
$	if f$extract(0,9,rec) .eqs. "Rmt Name:" .or. f$extract(0,13,rec) .eqs. -
			"Rmt BUS Name:"
$	then
$		crec = crec - "Rmt Name: " - "Rmt BUS Name: "
$		rmt_dev_name = f$element(0," ",crec)
$!!!		rmt_dev_type = f$element(3," ",crec)
$!!!		rmt_dev_addr = f$element(7," ",crec)
$! Don't display the local loopback path
$		if loc_dev_name .nes. "LCL"
$		then
$		    cnx_symbol = "+"
$		    if f$type(preferred_'chan_addr') .nes. "" then -
			cnx_symbol = "P"	!"*"
$		    write sys$output -
			f$fao("[!AS]: !AS !AS !AS :[!AS]",-
			this_node,loc_dev_name,cnx_symbol,rmt_dev_name,rmt_node)
$		    data_found = "T"
$		endif
$!		goto loop4
$	endif
$	goto loop4
$ eof4:
$	close tmp
$	delete 'temp_file'_list3;*	!!!
$	goto cleanup
$!
$! For VMS version 7.3 or later, we use SCACP instead of SDA to gather
$! SCA LAN path info.
$!
$ USE_SCACP:
$	close/nolog com
$	open/write com 'temp_file'_com
$	write com "$MCR SCACP SHOW CHANNEL"
$	close com
$ 'if_debug' type/page/header 'temp_file'_com	!!!
$	@'temp_file'_com/out='temp_file'_lis
$	delete 'temp_file'_com;*
$ 'if_debug' type/page/header 'temp_file'_lis	!!!
$	close/nolog lis
$	open/read lis 'temp_file'_lis
$! Element 0   1   2    3         4      5        6    7    8     9      10      11       12            13                 14
$!    Remote  LAN Dev Channel  Total    ECS     Priority       Buffer   Delay   Load    Total              ---- Most Recent ----
$!     Node   Loc Rmt  State  Errors   State    Cur  Mgt  Hops  Size    (uSec)  Class  Pkts(S+R)    CH Opened Time     CH Closed Time
$!     ----   --- ---  -----  ------   -----    ---  ---  ----  ----    ------  -----  ---------    --------------     --------------
$!    AIRTRN  EID EWB  Open       11  N(T,P,S)    0    0    2   1426    2765.0   100    3564434  30-JAN 13:42:02.02  30-JAN 13:40:42.96
$ scacp_loop:
$	read/end=scacp_eof lis rec
$ 'if_debug' show symbol rec	!!!
$	crec = f$edit(rec,"TRIM,COMPRESS")
$ 'if_debug' show symbol crec	!!!
$! Ignore heading lines
$!	if f$extract(0,22,crec) .eqs. "Remote LAN Dev Channel" then goto scacp_loop
$!	if f$extract(0,18,crec) .eqs. "Node Loc Rmt State" then goto scacp_loop
$!	if f$extract(0,4,crec) .eqs. "----" then goto scacp_loop
$! We can do the above easier in the Open-channel check below...
$	if f$extract(0,1,rec) .nes. " " then goto scacp_loop	!Ignore blank lines
$	rmt_node = f$element(0," ",crec)
$	loc_dev_name = f$element(1," ",crec)
$	rmt_dev_name = f$element(2," ",crec)
$	chan_state = f$element(3," ",crec)
$ 'if_debug' show symbol chan_state	!!!
$	if chan_state .nes. "Open" then goto scacp_loop	!Ignore closed channels
$ 'if_debug' show symbol rmt_node	!!!
$ 'if_debug' show symbol loc_dev_name	!!!
$ 'if_debug' show symbol rmt_dev_name	!!!
$!							!and heading lines
$!!!	err_cnt = f$element(4," ",crec)
$	ecs = f$element(5," ",crec)
$ 'if_debug' show symbol ecs	!!!
$!!!	buf_size = f$element(9," ",crec)
$	if loc_dev_name .eqs. "LCL" then goto scacp_loop	!Ignore LoCaL loopback path
$	write sys$output -
		f$fao("[!AS]: !AS !AS !AS :[!AS]",-
		this_node,loc_dev_name,ecs,rmt_dev_name,rmt_node)
$	data_found = "T"	!We found some data
$	goto scacp_loop
$ scacp_eof:
$	close lis
$	delete 'temp_file'_lis;*
$	goto cleanup
$ cleanup:
$! If we turned on elevated privileges, turn them back off again
$	if ELEVATED_PRIVS then set process/privilege=('remove_privs')
$	if data_found
$	then
$		write sys$output -
	    "Channel data collection successful on node ",this_node
$	else
$		write sys$output -
	    "*** Error: Data collection failed on node ",this_node," ***"
$	endif
$	exit

For V7.3 and later, here's an exaple of what we're parsing:

SKYWST PEA0 Channel Summary 13-FEB-2003 13:44:52.52:

  Remote  LAN Dev Channel  Total    ECS     Priority       Buffer   Delay   Load    Total              ---- Most Recent ----
   Node   Loc Rmt  State  Errors   State    Cur  Mgt  Hops  Size    (uSec)  Class  Pkts(S+R)    CH Opened Time     CH Closed Time
   ----   --- ---  -----  ------   -----    ---  ---  ----  ----    ------  -----  ---------    --------------     --------------
  AIRTRN  EID EWB  Open       11  N(T,P,S)    0    0    2   1426    2765.0   100    3564434  30-JAN 13:42:02.02  30-JAN 13:40:42.96
  AIRTRN  EWA EWB  Open       21  N(T,P,S)    0    0    2   1426    2459.8  1000    3548577  30-JAN 13:42:01.09  30-JAN 13:40:43.96
  AIRTRN  EIC EWA  Open       10  Y(T,P,F)    0    0    2   1426    1724.1   100    3578212  30-JAN 13:42:01.02  30-JAN 13:40:43.96
  AIRTRN  EIC EWB  Open       14  Y(T,P,F)    0    0    2   1426    2199.3   100    3558936  30-JAN 13:42:01.02  30-JAN 13:40:42.96
  AIRTRN  EID EWA  Open       11  Y(T,P,F)    0    0    2   1426    1789.9   100    3572388  30-JAN 13:42:00.02  30-JAN 13:40:43.96
  AIRTRN  EWA EWA  Open       23  N(T,P,S)    0    0    2   1426    2426.0  1000    3575790  30-JAN 13:42:00.02  30-JAN 13:40:43.96
  AIRTRN  EIC EIA  Open       16  Y(T,P,F)    0    0    2   1426    2209.3   100    3565099  30-JAN 13:41:28.14  30-JAN 13:40:42.96
  AIRTRN  EWA EIA  Open       16  N(T,P,S)    0    0    2   1426    2775.6  1000    3557914  30-JAN 13:41:28.14  30-JAN 13:40:43.96
  AIRTRN  EID EIA  Open       10  N(T,P,S)    0    0    2   1426    2941.6   100    3565662  30-JAN 13:41:28.14  30-JAN 13:40:43.96
  GALXY2  EID FWA  Open     1414  Y(T,P,F)    0    0    2   1426     903.9   100    7886485  24-JAN 15:54:49.36  (No time)         
  GALXY2  EIC FWA  Open     1339  Y(T,P,F)    0    0    2   1426     882.1   100    7934881  24-JAN 15:54:48.44  (No time)         
  GALXY2  EWA FWA  Open     1370  Y(T,P,F)    0    0    2   1426     880.2  1000    8104098  24-JAN 15:54:46.43  (No time)         
  GALXY1  EIC FWA  Open     7792  Y(T,P,F)    0    0    2   1426     908.1   100   21070002  24-JAN 15:50:51.33  (No time)         
  GALXY1  EWA FWA  Open     7133  Y(T,P,F)    0    0    2   1426     897.6  1000   20999257  24-JAN 15:50:51.33  (No time)         
  GALXY1  EID FWA  Open     9489  Y(T,P,F)    0    0    2   1426     873.2   100   23986640  24-JAN 15:50:50.63  (No time)         
  EASTRN  EIC FXB  Clsd       14  Y(T,P,F)    0    0    2   1426     250.0   100    1710898  24-JAN 15:50:52.30  03-FEB 09:23:53.05
  EASTRN  EID FXB  Clsd       11  Y(T,P,F)    0    0    2   1426     250.0   100    1713999  24-JAN 15:50:52.31  03-FEB 09:23:53.05
  EASTRN  EWA FXB  Clsd       18  Y(T,P,F)    0    0    2   1426     250.0  1000    1669035  24-JAN 15:50:52.31  03-FEB 09:23:53.05
  EASTRN  EID FXA  Clsd       11  Y(T,P,F)    0    0    2   1426     250.0   100    1706867  24-JAN 15:50:20.33  03-FEB 09:23:52.05
  EASTRN  EIC FXA  Clsd       14  Y(T,P,F)    0    0    2   1426     250.0   100    1713628  24-JAN 15:50:20.72  03-FEB 09:23:52.05
  EASTRN  EWA FXA  Clsd       19  Y(T,P,F)    0    0    2   1426     250.0  1000    1733045  24-JAN 15:50:20.33  03-FEB 09:23:52.05
  WSTERN  EIC FWA  Open     8152  Y(T,P,F)    0    0    2   1426     892.4   100   20570415  11-FEB 20:12:55.94  11-FEB 20:10:16.80
  WSTERN  EWA FWA  Open     8200  Y(T,P,F)    0    0    2   1426     887.3  1000   23904560  11-FEB 20:12:55.67  11-FEB 20:10:16.80
  WSTERN  EID FWA  Open     7951  Y(T,P,F)    0    0    2   1426     857.3   100   20372524  11-FEB 20:12:55.67  11-FEB 20:10:16.80
  SKYKNG  EIC EWA  Open     3134  Y(T,P,F)    0    0    2   1412     863.6   100   11473487  24-JAN 15:46:50.99  (No time)         
  SKYKNG  EWA EWA  Open     2917  Y(T,P,F)    0    0    2   1412     867.4  1000   12030003  24-JAN 15:46:49.99  (No time)         
  SKYKNG  EID EWA  Open     3091  Y(T,P,F)    0    0    2   1412     873.3   100    8531443  24-JAN 15:46:49.99  (No time)         
  SKYKNG  EWA EIA  Open     2623  Y(T,P,F)    0    0    2   1412     926.6  1000   11131795  24-JAN 15:46:21.30  (No time)         
  SKYKNG  EIC EIA  Open     2964  Y(T,P,F)    0    0    2   1412     891.8   100    8451465  24-JAN 15:46:21.30  (No time)         
  SKYKNG  EIC EIB  Open     3002  Y(T,P,F)    0    0    2   1412     890.7   100    6323114  24-JAN 15:46:21.30  (No time)         
  SKYKNG  EID EIA  Open     3031  Y(T,P,F)    0    0    2   1412     888.7   100    8711857  24-JAN 15:46:21.30  (No time)         
  SKYKNG  EID EIB  Open     3010  Y(T,P,F)    0    0    2   1412     921.8   100    6395601  24-JAN 15:46:21.30  (No time)         
  SKYKNG  EWA EIB  Open     2558  Y(T,P,F)    0    0    2   1412     936.2  1000    5670051  24-JAN 15:46:21.30  (No time)         
  BRANIF  EID FXA  Open      336  Y(T,P,F)    0    0    2   1412    1108.0   100  164191163  24-JAN 15:40:09.38  (No time)         
  BRANIF  EIC FXA  Open      331  Y(T,P,F)    0    0    2   1412    1118.2   100  166917467  24-JAN 15:40:09.38  (No time)         
  BRANIF  EWA FXA  Open      301  Y(T,P,F)    0    0    2   1412    1155.1  1000  174811135  24-JAN 15:40:09.25  (No time)         
  SKYHWK  EWA EWA  Open      286  Y(T,P,F)    0    0    2   1412     870.2  1000    5403160  24-JAN 15:39:58.30  (No time)         
  SKYHWK  EID EWA  Open      345  Y(T,P,F)    0    0    2   1412     848.3   100    4708916  24-JAN 15:39:57.30  (No time)         
  SKYHWK  EIC EWA  Open      311  Y(T,P,F)    0    0    2   1412     829.7   100    4531553  24-JAN 15:39:57.30  (No time)         
  SKYHWK  EID EIA  Open      312  Y(T,P,F)    0    0    2   1412     847.0   100    5097807  24-JAN 15:39:29.25  (No time)         
  SKYHWK  EWA EIA  Open      279  Y(T,P,F)    0    0    2   1412     851.0  1000    5924012  24-JAN 15:39:29.25  (No time)         
  SKYHWK  EWA EIB  Open      288  Y(T,P,F)    0    0    2   1412     870.3  1000    3997676  24-JAN 15:39:29.25  (No time)         
  SKYHWK  EID EIB  Open      314  Y(T,P,F)    0    0    2   1412     847.3   100    3957100  24-JAN 15:39:29.25  (No time)         
  SKYHWK  EIC EIA  Open      338  Y(T,P,F)    0    0    2   1412     853.3   100    4112203  24-JAN 15:39:28.49  (No time)         
  SKYHWK  EIC EIB  Open      336  Y(T,P,F)    0    0    2   1412     837.8   100    4545021  24-JAN 15:39:28.49  (No time)         
  SKYWAY  EIC EWA  Open      232  Y(T,P,F)    0    0    2   1426    2089.7   100    9110100  24-JAN 15:34:24.38  (No time)         
  SKYWAY  EWA EWA  Open      203  N(T,P,S)    0    0    2   1426    2404.5  1000    9107547  24-JAN 15:34:23.38  (No time)         
  SKYWAY  EID EWA  Open      219  Y(T,P,F)    0    0    2   1426    2225.3   100    9098348  24-JAN 15:34:22.38  (No time)         
  SKYWAY  EIC EIA  Open      218  Y(T,P,F)    0    0    2   1426    1859.5   100    9124475  24-JAN 15:34:22.23  (No time)         
  SKYWAY  EWA EIA  Open      210  Y(T,P,F)    0    0    2   1426    2002.1  1000    9022737  24-JAN 15:34:22.08  (No time)         
  SKYWAY  EID EIA  Open      192  N(T,P,S)    0    0    2   1426    2384.2   100    9060363  24-JAN 15:34:21.23  (No time)         
  SKYWAY  EID EIB  Open      226  Y(T,P,F)    0    0    2   1426    2146.2   100    9099512  24-JAN 15:33:53.53  (No time)         
  SKYWAY  EWA EIB  Open      228  Y(T,P,F)    0    0    2   1426    1626.0  1000    9104862  24-JAN 15:33:53.35  (No time)         
  SKYWAY  EIC EIB  Open      227  N(T,P,S)    0    0    2   1426    2484.6   100    9153634  24-JAN 15:33:53.35  (No time)         
  EMPIRE  EWA EWA  Open       20  N(T,P,S)    0    0    2   1426    3928.5  1000   20208305  24-JAN 15:31:18.51  (No time)         
  EMPIRE  EIC EWA  Open       21  N(T,P,S)    0    0    2   1426    4064.5   100    5682215  24-JAN 15:31:18.51  (No time)         
  EMPIRE  EID EWA  Open       21  N(T,P,S)    0    0    2   1426    4207.8   100    5673730  24-JAN 15:31:18.23  (No time)         
  EMPIRE  EWA EIA  Open       27  N(T,P,S)    0    0    2   1426    4144.9  1000    5650652  24-JAN 15:30:51.23  (No time)         
  EMPIRE  EWA EIB  Open       25  N(T,P,S)    0    0    2   1426    4717.3  1000    5612824  24-JAN 15:30:51.23  (No time)         
  EMPIRE  EID EIB  Open       15  N(T,P,S)    0    0    2   1426    4113.4   100    4985852  24-JAN 15:30:51.09  (No time)         
  EMPIRE  EID EIA  Open       28  Y(T,P,F)    0    0    2   1426    4036.4   100    4897868  24-JAN 15:30:51.09  (No time)         
  EMPIRE  EIC EIA  Open       23  N(T,P,S)    0    0    2   1426    3905.7   100    4940786  24-JAN 15:30:50.23  (No time)         
  EMPIRE  EIC EIB  Open       18  N(T,P,S)    0    0    2   1426    4610.5   100    4930625  24-JAN 15:30:50.23  (No time)         
  SKYWST  EWA EIC  Open        4  N(T,P,S)    0    0    2   1426     898.1  1000    3047272  24-JAN 15:27:42.31  (No time)         
  SKYWST  EID EWA  Open        3  N(T,P,S)    0    0    2   1426     900.2   100    3042424  24-JAN 15:27:42.31  (No time)         
  SKYWST  EIC EWA  Open        3  N(T,P,S)    0    0    2   1426     887.8   100    3042424  24-JAN 15:27:42.31  (No time)         
  SKYWST  EWA EID  Open        4  N(T,P,S)    0    0    2   1426     898.0  1000    3032596  24-JAN 15:27:42.31  (No time)         
  SKYWST  EIC EID  Open        1  N(T,P,S)    0    0    2   1426     871.5   100    3032669  24-JAN 15:27:10.29  (No time)         
  SKYWST  EID EIC  Open        1  N(T,P,S)    0    0    2   1426     872.5   100    3047348  24-JAN 15:27:10.29  (No time)         
  SKYWST  LCL LCL  Open        1  Y(T,P,F)    0    0    2   1426     314.9     0    3029364  24-JAN 15:27:08.87  (No time)         


